@@ -23,7 +23,6 @@ Authors : Jean-Francois Dufayard & Stephane Guindon.
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 /*
 * Check every testable branch of the tree,
 * check for NNIs, optimizing 5 branches,
@@ -41,15 +40,15 @@ int Check_NNI_Five_Branches(t_tree *tree)
   phydbl init_lnL;
 
   init_lnL     = UNLIKELY;
-  better_found = 1;
+  better_found = YES;
   
   //While there is at least one NNI to do...
-  while(better_found)
+  while(better_found == YES)
     {
       Update_Dirs(tree);
-      
+            
       //Interface output
-      if((tree->mod->s_opt->print) && (!tree->io->quiet)) PhyML_Printf("\n\n. Checking for NNIs, optimizing five branches...\n");
+      if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf("\n\n. Checking for NNIs, optimizing five branches...\n");
       
       better_found  =  0;
       result        = -1;
@@ -62,8 +61,9 @@ int Check_NNI_Five_Branches(t_tree *tree)
       init_lnL = Lk(NULL,tree);
       MIXT_Set_Alias_Subpatt(NO,tree);
       
+      
       //For every branch
-      For(i,2*tree->n_otu-3)
+      for(i=0;i<2*tree->n_otu-3;++i)
         {
           //if this branch is not terminal
           if((!tree->a_edges[i]->left->tax) && (!tree->a_edges[i]->rght->tax))
@@ -80,7 +80,7 @@ int Check_NNI_Five_Branches(t_tree *tree)
                   {
                     if((tree->a_edges[i]->nni->lk0 - tree->a_edges[i]->nni->lk1) < best_gain)
                       {
-                        better_found = 1;
+                        better_found = YES;
                         best_edge    = i;
                         best_gain    = tree->a_edges[i]->nni->lk0-tree->a_edges[i]->nni->lk1;
                         best_config  = 1;
@@ -91,7 +91,7 @@ int Check_NNI_Five_Branches(t_tree *tree)
                   {
                     if((tree->a_edges[i]->nni->lk0 - tree->a_edges[i]->nni->lk2) < best_gain)
                       {
-                        better_found = 1;
+                        better_found = YES;
                         best_edge    = i;
                         best_gain    = tree->a_edges[i]->nni->lk0-tree->a_edges[i]->nni->lk2;
                         best_config  = 2;
@@ -102,7 +102,7 @@ int Check_NNI_Five_Branches(t_tree *tree)
                   {
                     if((tree->a_edges[i]->nni->lk2 - tree->a_edges[i]->nni->lk1) < best_gain)
                       {
-                        better_found = 1;
+                        better_found = YES;
                         best_edge    = i;
                         best_gain    = tree->a_edges[i]->nni->lk2-tree->a_edges[i]->nni->lk1;
                         best_config  = 1;
@@ -113,7 +113,7 @@ int Check_NNI_Five_Branches(t_tree *tree)
                   {
                     if((tree->a_edges[i]->nni->lk1 - tree->a_edges[i]->nni->lk2) < best_gain)
                       {
-                        better_found = 1;
+                        better_found = YES;
                         best_edge    = i;
                         best_gain    = tree->a_edges[i]->nni->lk1-tree->a_edges[i]->nni->lk2;
                         best_config  = 2;
@@ -129,15 +129,16 @@ int Check_NNI_Five_Branches(t_tree *tree)
             }
         }
       
+      
       if((tree->c_lnL < init_lnL - tree->mod->s_opt->min_diff_lk_local) || (tree->c_lnL > init_lnL + tree->mod->s_opt->min_diff_lk_local))
         {
-          PhyML_Printf("\n\n== tree->c_lnL = %f init_lnL = %f.",tree->c_lnL,init_lnL);
-          PhyML_Printf("\n== Err. in file %s at line %d\n\n.\n",__FILE__,__LINE__);
+          PhyML_Fprintf(stderr,"\n\n== tree->c_lnL = %f init_lnL = %f.",tree->c_lnL,init_lnL);
+          PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n\n.\n",__FILE__,__LINE__);
           Warn_And_Exit("\n");
         }
       
       //Don't do any NNI if the user doesn't want to optimize topology
-      if(!tree->mod->s_opt->opt_topo) better_found = 0;
+      if(!tree->mod->s_opt->opt_topo) better_found = NO;
 /*       if(FABS(best_gain) <= tree->mod->s_opt->min_diff_lk_move) better_found = 0; */
 
       //If there is one swap to do, make the best one.
@@ -151,12 +152,12 @@ int Check_NNI_Five_Branches(t_tree *tree)
 
           if(tree->c_lnL < init_lnL)
             {
-              PhyML_Printf("\n\n== tree->c_lnL = %f init_lnL = %f.",tree->c_lnL,init_lnL);
-              PhyML_Printf("\n== Err. in file %s at line %d\n\n.\n",__FILE__,__LINE__);
+              PhyML_Fprintf(stderr,"\n\n== tree->c_lnL = %f init_lnL = %f.",tree->c_lnL,init_lnL);
+              PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n\n.\n",__FILE__,__LINE__);
               Exit("\n");
             }
 
-          if((tree->mod->s_opt->print) && (!tree->io->quiet)) Print_Lk(tree,"[Topology           ]");
+          if(tree->verbose > VL0 && tree->io->quiet == NO) Print_Lk(tree,"[Topology           ]");
 
           if(FABS(tree->c_lnL - init_lnL) < tree->mod->s_opt->min_diff_lk_move) return 0;
           return 1;
@@ -205,6 +206,7 @@ void aLRT(t_tree *tree)
   Lk(NULL,tree);
   //  Print_All_Edge_Likelihoods(tree);
   MIXT_Set_Alias_Subpatt(NO,tree);
+  Update_Dirs(tree);
   
   For(i,2*tree->n_otu-3)
     if((!tree->a_edges[i]->left->tax) && (!tree->a_edges[i]->rght->tax))
@@ -293,9 +295,9 @@ int Compute_Likelihood_Ratio_Test(t_edge *tested_edge, t_tree *tree)
               
               logK = 1. - MAX(MAX(tested_edge->nni->lk0,tested_edge->nni->lk1),tested_edge->nni->lk2);
               
-              Kp0 = EXP(tested_edge->nni->lk0+logK);
-              Kp1 = EXP(tested_edge->nni->lk1+logK);
-              Kp2 = EXP(tested_edge->nni->lk2+logK);
+              Kp0 = exp(tested_edge->nni->lk0+logK);
+              Kp1 = exp(tested_edge->nni->lk1+logK);
+              Kp2 = exp(tested_edge->nni->lk2+logK);
               tested_edge->ratio_test = Kp0/(Kp0+Kp1+Kp2);              
             }
         }
@@ -347,19 +349,21 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   int site;
   t_node *v1,*v2,*v3,*v4;
   t_edge *e1,*e2,*e3,*e4;
-  phydbl *len_e1,*len_e2,*len_e3,*len_e4;
+  scalar_dbl *len_e1,*len_e2,*len_e3,*len_e4;
+  scalar_dbl *var_e1,*var_e2,*var_e3,*var_e4;
   phydbl lk0, lk1, lk2;
-  phydbl *bl_init;
-  phydbl l_infa, l_infb;
+  scalar_dbl *l_init,*v_init;
   phydbl lk_init, lk_temp;
   int i;
-  int result,counter,wei;
+  int result;
   void *buff;
 
   result = 0;
 
+
   /*! Initialization */
-  bl_init            = MIXT_Get_Lengths_Of_This_Edge(b_fcus,tree);
+  l_init             = Duplicate_Scalar_Dbl(b_fcus->l);
+  v_init             = Duplicate_Scalar_Dbl(b_fcus->l_var);
   lk_init            = tree->c_lnL;
   lk_temp            = UNLIKELY;
   b_fcus->nni->score = .0;
@@ -374,12 +378,13 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
 
   if(v1->num < v2->num)
     {
-      PhyML_Printf("\n== Err. in file %s at line %d\n\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n\n",__FILE__,__LINE__);
       Exit("");
     }
+
   if(v3->num < v4->num)
     {
-      PhyML_Printf("\n== Err. in file %s at line %d\n\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n\n",__FILE__,__LINE__);
       Exit("");
     }
 
@@ -390,10 +395,14 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   e4 = b_fcus->rght->b[b_fcus->r_v2];
 
   /*! record initial branch lengths */
-  len_e1 = MIXT_Get_Lengths_Of_This_Edge(e1,tree);
-  len_e2 = MIXT_Get_Lengths_Of_This_Edge(e2,tree);
-  len_e3 = MIXT_Get_Lengths_Of_This_Edge(e3,tree);
-  len_e4 = MIXT_Get_Lengths_Of_This_Edge(e4,tree);
+  len_e1 = Duplicate_Scalar_Dbl(e1->l);
+  len_e2 = Duplicate_Scalar_Dbl(e2->l);
+  len_e3 = Duplicate_Scalar_Dbl(e3->l);
+  len_e4 = Duplicate_Scalar_Dbl(e4->l);
+  var_e1 = Duplicate_Scalar_Dbl(e1->l_var);
+  var_e2 = Duplicate_Scalar_Dbl(e2->l_var);
+  var_e3 = Duplicate_Scalar_Dbl(e3->l_var);
+  var_e4 = Duplicate_Scalar_Dbl(e4->l_var);
 
   /*! Optimize branch lengths and update likelihoods for
     the original configuration.
@@ -402,35 +411,28 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
     {
       lk0 = lk_temp;
 
-      For(i,3) //a branch has three nodes
-        if(b_fcus->left->v[i] != b_fcus->rght)//Only consider left_1 and left_2
+      for(i=0;i<3;i++) //a branch has three nodes
+        if(b_fcus->left->v[i] != b_fcus->rght) //Only consider left_1 and left_2
           {
-            Update_P_Lk(tree,b_fcus->left->b[i],b_fcus->left);
-            l_infa  = 10.;
-            l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->left->b[i]->l->v);
-            lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus->left->b[i],tree);
+            Update_Partial_Lk(tree,b_fcus->left->b[i],b_fcus->left);
+            lk_temp = Br_Len_Opt(b_fcus->left->b[i],tree);
           }
 
-      Update_P_Lk(tree,b_fcus,b_fcus->left);
+      Update_Partial_Lk(tree,b_fcus,b_fcus->left);
+      lk_temp = Br_Len_Opt(b_fcus,tree);
 
-      l_infa  = 10.;
-      l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->l->v);
-      lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus,tree);
-
-      For(i,3)
-        if(b_fcus->rght->v[i] != b_fcus->left)//Only consider right_1 and right_2
+      for(i=0;i<3;i++)
+        if(b_fcus->rght->v[i] != b_fcus->left) //Only consider right_1 and right_2
           {
-            Update_P_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
-            l_infa  = 10.;
-            l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->rght->b[i]->l->v);
-            lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus->rght->b[i],tree);
+            Update_Partial_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
+            lk_temp = Br_Len_Opt(b_fcus->rght->b[i],tree);
           }
-      Update_P_Lk(tree,b_fcus,b_fcus->rght);
+      Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
 
       if(lk_temp < lk0 - tree->mod->s_opt->min_diff_lk_local)
         {
-          PhyML_Printf("\n== lk_temp = %f lk0 = %f\n",lk_temp,lk0);
-          PhyML_Printf("\n== Err. in file %s at line %d\n\n",__FILE__,__LINE__);
+          PhyML_Fprintf(stderr,"\n== lk_temp = %f lk0 = %f\n",lk_temp,lk0);
+          PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n\n",__FILE__,__LINE__);
           Exit("\n");
         }
     }
@@ -438,48 +440,45 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   //until no significative improvement is detected
   
   lk0 = tree->c_lnL;
-
+ 
   buff = (t_tree *)tree;
   do
     {
-      counter=0;
-      For(site,tree->n_pattern)
-        {
-          wei=0;
-          For(wei,tree->data->wght[site])
-            {
-              tree->log_lks_aLRT[0][counter]= tree->c_lnL_sorted[site] / tree->data->wght[site];
-              counter++;
-            }
-        }
+      for(site=0;site<tree->n_pattern;site++) tree->log_lks_aLRT[0][site] = tree->c_lnL_sorted[site] * tree->data->wght[site];
       tree = tree->next_mixt;
     }
   while(tree);
 
   tree = (t_tree *)buff;
+
   /*! Go back to initial branch lengths */
-  MIXT_Set_Lengths_Of_This_Edge(len_e1,e1,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e2,e2,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e3,e3,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e4,e4,tree);
-  MIXT_Set_Lengths_Of_This_Edge(bl_init,b_fcus,tree);
+  Copy_Scalar_Dbl(len_e1,e1->l);
+  Copy_Scalar_Dbl(len_e2,e2->l);
+  Copy_Scalar_Dbl(len_e3,e3->l);
+  Copy_Scalar_Dbl(len_e4,e4->l);
+  Copy_Scalar_Dbl(l_init,b_fcus->l);
+  Copy_Scalar_Dbl(var_e1,e1->l_var);
+  Copy_Scalar_Dbl(var_e2,e2->l_var);
+  Copy_Scalar_Dbl(var_e3,e3->l_var);
+  Copy_Scalar_Dbl(var_e4,e4->l_var);
+  Copy_Scalar_Dbl(v_init,b_fcus->l_var);
+
   Update_PMat_At_Given_Edge(e1,tree);
   Update_PMat_At_Given_Edge(e2,tree);
   Update_PMat_At_Given_Edge(e3,tree);
   Update_PMat_At_Given_Edge(e4,tree);
   Update_PMat_At_Given_Edge(b_fcus,tree);
 
-
   /* Sanity check */
   MIXT_Set_Alias_Subpatt(YES,tree);
-  Update_P_Lk(tree,b_fcus,b_fcus->rght);
-  Update_P_Lk(tree,b_fcus,b_fcus->left);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->left);
   lk_temp = Lk(b_fcus,tree);
   MIXT_Set_Alias_Subpatt(NO,tree);
   if((lk_temp > lk_init + tree->mod->s_opt->min_diff_lk_local) || (lk_temp < lk_init - tree->mod->s_opt->min_diff_lk_local))
     {
-      PhyML_Printf("\n== lk_temp = %f lk_init = %f",lk_temp,lk_init);
-      PhyML_Printf("\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n== lk_temp = %f lk_init = %f",lk_temp,lk_init);
+      PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
       Exit("\n");
     }
 
@@ -488,18 +487,18 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   Swap(v2,b_fcus->left,b_fcus->rght,v3,tree);
   MIXT_Set_Alias_Subpatt(YES,tree);
   Set_Both_Sides(YES,tree);
-  Update_P_Lk(tree,b_fcus,b_fcus->left);
-  Update_P_Lk(tree,b_fcus,b_fcus->rght);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->left);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
   lk1 = Lk(b_fcus,tree);
   MIXT_Set_Alias_Subpatt(NO,tree);
 
-  For(i,3)
+  for(i=0;i<3;i++)
     if(b_fcus->left->v[i] != b_fcus->rght)
-      Update_P_Lk(tree,b_fcus->left->b[i],b_fcus->left);
+      Update_Partial_Lk(tree,b_fcus->left->b[i],b_fcus->left);
   
-  For(i,3)
+  for(i=0;i<3;i++)
     if(b_fcus->rght->v[i] != b_fcus->left)
-      Update_P_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
+      Update_Partial_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
   
   
   /*! Optimize branch lengths and update likelihoods */
@@ -508,38 +507,29 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
     {
       lk1 = lk_temp;
       
-      For(i,3)
+      for(i=0;i<3;i++)
         if(b_fcus->left->v[i] != b_fcus->rght)
           {
-            Update_P_Lk(tree,b_fcus->left->b[i],b_fcus->left);
-            
-            l_infa  = 10.;
-            l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->left->b[i]->l->v);
-            lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus->left->b[i],tree);
+            Update_Partial_Lk(tree,b_fcus->left->b[i],b_fcus->left);
+            lk_temp = Br_Len_Opt(b_fcus->left->b[i],tree);
           }
       
-      Update_P_Lk(tree,b_fcus,b_fcus->left);
+      Update_Partial_Lk(tree,b_fcus,b_fcus->left);
+      lk_temp = Br_Len_Opt(b_fcus,tree);
       
-      l_infa  = 10.;
-      l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->l->v);
-      lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus,tree);
-      
-      For(i,3)
+      for(i=0;i<3;i++)
         if(b_fcus->rght->v[i] != b_fcus->left)
           {
-            Update_P_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
-            
-            l_infa  = 10.;
-            l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->rght->b[i]->l->v);
-            lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus->rght->b[i],tree);
+            Update_Partial_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
+            lk_temp = Br_Len_Opt(b_fcus->rght->b[i],tree);
           }
       
-      Update_P_Lk(tree,b_fcus,b_fcus->rght);
+      Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
       
       if(lk_temp < lk1 - tree->mod->s_opt->min_diff_lk_local)
         {
-          PhyML_Printf("\n== lk_temp = %f lk1 = %f\n",lk_temp,lk1);
-          PhyML_Printf("\n== Err in file %s at line %d\n\n",__FILE__,__LINE__);
+          PhyML_Fprintf(stderr,"\n== lk_temp = %f lk1 = %f\n",lk_temp,lk1);
+          PhyML_Fprintf(stderr,"\n== Err in file %s at line %d\n\n",__FILE__,__LINE__);
           Warn_And_Exit("");
         }
     }
@@ -553,29 +543,25 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   buff = (t_tree *)tree;
   do
     {
-      counter=0;
-      For(site,tree->n_pattern)
-        {
-          wei=0;
-          For(wei,tree->data->wght[site])
-            {
-              tree->log_lks_aLRT[1][counter]= tree->c_lnL_sorted[site] / tree->data->wght[site];
-              counter++;
-            }
-        }
+      for(site=0;site<tree->n_pattern;site++) tree->log_lks_aLRT[1][site]= tree->c_lnL_sorted[site] * tree->data->wght[site];
       tree = tree->next_mixt;
     }
   while(tree);
   tree = (t_tree *)buff;
       
-
   Swap(v3,b_fcus->left,b_fcus->rght,v2,tree);
+
   /*! Go back to initial branch lengths */
-  MIXT_Set_Lengths_Of_This_Edge(len_e1,e1,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e2,e2,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e3,e3,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e4,e4,tree);
-  MIXT_Set_Lengths_Of_This_Edge(bl_init,b_fcus,tree);
+  Copy_Scalar_Dbl(len_e1,e1->l);
+  Copy_Scalar_Dbl(len_e2,e2->l);
+  Copy_Scalar_Dbl(len_e3,e3->l);
+  Copy_Scalar_Dbl(len_e4,e4->l);
+  Copy_Scalar_Dbl(l_init,b_fcus->l);
+  Copy_Scalar_Dbl(var_e1,e1->l_var);
+  Copy_Scalar_Dbl(var_e2,e2->l_var);
+  Copy_Scalar_Dbl(var_e3,e3->l_var);
+  Copy_Scalar_Dbl(var_e4,e4->l_var);
+  Copy_Scalar_Dbl(v_init,b_fcus->l_var);
   
   Update_PMat_At_Given_Edge(e1,tree);
   Update_PMat_At_Given_Edge(e2,tree);
@@ -586,14 +572,14 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   
   /* Sanity check */
   MIXT_Set_Alias_Subpatt(YES,tree);
-  Update_P_Lk(tree,b_fcus,b_fcus->rght);
-  Update_P_Lk(tree,b_fcus,b_fcus->left);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->left);
   lk_temp = Lk(b_fcus,tree);
   MIXT_Set_Alias_Subpatt(NO,tree);
   if((lk_temp > lk_init + tree->mod->s_opt->min_diff_lk_local) || (lk_temp < lk_init - tree->mod->s_opt->min_diff_lk_local))
     {
-      PhyML_Printf("\n== lk_temp = %f lk_init = %f",lk_temp,lk_init);
-      PhyML_Printf("\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n== lk_temp = %f lk_init = %f",lk_temp,lk_init);
+      PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
       Warn_And_Exit("");
     }
     
@@ -601,18 +587,18 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   Swap(v2,b_fcus->left,b_fcus->rght,v4,tree);
   Set_Both_Sides(YES,tree);
   MIXT_Set_Alias_Subpatt(YES,tree);
-  Update_P_Lk(tree,b_fcus,b_fcus->left);
-  Update_P_Lk(tree,b_fcus,b_fcus->rght);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->left);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
   lk2 = Lk(b_fcus,tree);
   MIXT_Set_Alias_Subpatt(NO,tree);
   
-  For(i,3)
+  for(i=0;i<3;i++)
     if(b_fcus->left->v[i] != b_fcus->rght)
-      Update_P_Lk(tree,b_fcus->left->b[i],b_fcus->left);
+      Update_Partial_Lk(tree,b_fcus->left->b[i],b_fcus->left);
   
-  For(i,3)
+  for(i=0;i<3;i++)
     if(b_fcus->rght->v[i] != b_fcus->left)
-      Update_P_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
+      Update_Partial_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
   
   /*! Optimize branch lengths and update likelihoods */
   lk_temp = UNLIKELY;
@@ -620,59 +606,50 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
     {
       lk2 = lk_temp;
       
-      For(i,3)
+      for(i=0;i<3;i++)
         if(b_fcus->left->v[i] != b_fcus->rght)
           {
-            Update_P_Lk(tree,b_fcus->left->b[i],b_fcus->left);
-            
-            l_infa  = 10.;
-            l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->left->b[i]->l->v);
-            lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus->left->b[i],tree);
+            Update_Partial_Lk(tree,b_fcus->left->b[i],b_fcus->left);
+            lk_temp = Br_Len_Opt(b_fcus->left->b[i],tree);
             
             if(lk_temp < lk2 - tree->mod->s_opt->min_diff_lk_local)
               {
-                PhyML_Printf("\n== l_infa: %f l_infb: %f l: %f var:%f",l_infa,l_infb,b_fcus->left->b[i]->l->v,b_fcus->left->b[i]->l_var->v);
-                PhyML_Printf("\n== lk_temp = %f lk2 = %f",lk_temp,lk2);
-                PhyML_Printf("\n== Err. in file %s at line %d",__FILE__,__LINE__);
+                PhyML_Fprintf(stderr,"\n== l: %f var:%f",b_fcus->left->b[i]->l->v,b_fcus->left->b[i]->l_var->v);
+                PhyML_Fprintf(stderr,"\n== lk_temp = %f lk2 = %f",lk_temp,lk2);
+                PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d",__FILE__,__LINE__);
                 Exit("\n");
               }
           }
-      Update_P_Lk(tree,b_fcus,b_fcus->left);
-      
-      l_infa  = 10.;
-      l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->l->v);
-      lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus,tree);
+      Update_Partial_Lk(tree,b_fcus,b_fcus->left);
+      lk_temp = Br_Len_Opt(b_fcus,tree);
       
       if(lk_temp < lk2 - tree->mod->s_opt->min_diff_lk_local)
         {
-          PhyML_Printf("\n== l_infa: %f l_infb: %f l: %f var:%f",l_infa,l_infb,b_fcus->l->v,b_fcus->l_var->v);
-          PhyML_Printf("\n== lk_temp = %f lk2 = %f",lk_temp,lk2);
-          PhyML_Printf("\n== Err. in file %s at line %d",__FILE__,__LINE__);
+          PhyML_Fprintf(stderr,"\n== l: %f var:%f",b_fcus->l->v,b_fcus->l_var->v);
+          PhyML_Fprintf(stderr,"\n== lk_temp = %f lk2 = %f",lk_temp,lk2);
+          PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d",__FILE__,__LINE__);
           Exit("\n");
         }
       
-      For(i,3)
+      for(i=0;i<3;i++)
         if(b_fcus->rght->v[i] != b_fcus->left)
           {
-            Update_P_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
-            
-            l_infa  = 10.;
-            l_infb  = MAX(0.0,tree->mod->l_min/b_fcus->rght->b[i]->l->v);
-            lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus->rght->b[i],tree);
+            Update_Partial_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
+            lk_temp = Br_Len_Opt(b_fcus->rght->b[i],tree);
             
             if(lk_temp < lk2 - tree->mod->s_opt->min_diff_lk_local)
               {
                 Set_Both_Sides(YES,tree);
                 Lk(b_fcus,tree);
                 Check_Lk_At_Given_Edge(YES,tree);
-                PhyML_Printf("\n== l_infa: %f l_infb: %f l: %f var:%f",l_infa,l_infb,b_fcus->rght->b[i]->l->v,b_fcus->rght->b[i]->l_var->v);
-                PhyML_Printf("\n== lk_temp = %f lk2 = %f",lk_temp,lk2);
-                PhyML_Printf("\n== Err. in file %s at line %d",__FILE__,__LINE__);
+                PhyML_Fprintf(stderr,"\n== l: %f var:%f",b_fcus->rght->b[i]->l->v,b_fcus->rght->b[i]->l_var->v);
+                PhyML_Fprintf(stderr,"\n== lk_temp = %f lk2 = %f",lk_temp,lk2);
+                PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d",__FILE__,__LINE__);
                 Exit("\n");
               }
           }
       
-      Update_P_Lk(tree,b_fcus,b_fcus->rght);
+      Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
       
     }
   while(FABS(lk_temp-lk2) > tree->mod->s_opt->min_diff_lk_global);
@@ -684,16 +661,7 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   buff = (t_tree *)tree;
   do
     {
-      counter=0;
-      For(site,tree->n_pattern)
-        {
-          wei=0;
-          For(wei,tree->data->wght[site])
-            {
-              tree->log_lks_aLRT[2][counter]= tree->c_lnL_sorted[site] / tree->data->wght[site];
-              counter++;
-            }
-        }
+      for(site=0;site<tree->n_pattern;site++) tree->log_lks_aLRT[2][site]= tree->c_lnL_sorted[site] * tree->data->wght[site];
       tree = tree->next_mixt;
     }
   while(tree);
@@ -705,11 +673,17 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   /***********/
 
   /*! restore the initial branch length values */
-  MIXT_Set_Lengths_Of_This_Edge(len_e1,e1,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e2,e2,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e3,e3,tree);
-  MIXT_Set_Lengths_Of_This_Edge(len_e4,e4,tree);
-  MIXT_Set_Lengths_Of_This_Edge(bl_init,b_fcus,tree);
+  Copy_Scalar_Dbl(len_e1,e1->l);
+  Copy_Scalar_Dbl(len_e2,e2->l);
+  Copy_Scalar_Dbl(len_e3,e3->l);
+  Copy_Scalar_Dbl(len_e4,e4->l);
+  Copy_Scalar_Dbl(l_init,b_fcus->l);
+  Copy_Scalar_Dbl(var_e1,e1->l_var);
+  Copy_Scalar_Dbl(var_e2,e2->l_var);
+  Copy_Scalar_Dbl(var_e3,e3->l_var);
+  Copy_Scalar_Dbl(var_e4,e4->l_var);
+  Copy_Scalar_Dbl(v_init,b_fcus->l_var);
+
 
   /*! recompute likelihoods */
   Update_PMat_At_Given_Edge(e1,tree);
@@ -719,16 +693,16 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
   Update_PMat_At_Given_Edge(b_fcus,tree);
 
   MIXT_Set_Alias_Subpatt(YES,tree);
-  Update_P_Lk(tree,b_fcus,b_fcus->left);
-  Update_P_Lk(tree,b_fcus,b_fcus->rght);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->left);
+  Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
 
-  For(i,3)
+  for(i=0;i<3;i++)
     if(b_fcus->left->v[i] != b_fcus->rght)
-      Update_P_Lk(tree,b_fcus->left->b[i],b_fcus->left);
+      Update_Partial_Lk(tree,b_fcus->left->b[i],b_fcus->left);
 
-  For(i,3)
+  for(i=0;i<3;i++)
     if(b_fcus->rght->v[i] != b_fcus->left)
-      Update_P_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
+      Update_Partial_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
 
   MIXT_Set_Alias_Subpatt(NO,tree);
 
@@ -736,8 +710,8 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
 
   if((lk_temp > lk_init + tree->mod->s_opt->min_diff_lk_local) || (lk_temp < lk_init - tree->mod->s_opt->min_diff_lk_local))
     {
-      PhyML_Printf("\n== lk_temp = %f lk_init = %f",lk_temp,lk_init);
-      PhyML_Printf("\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n== lk_temp = %f lk_init = %f",lk_temp,lk_init);
+      PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
       Warn_And_Exit("");
     }
 
@@ -769,11 +743,16 @@ int NNI_Neigh_BL(t_edge *b_fcus, t_tree *tree)
     }
 
 
-  Free(len_e1);
-  Free(len_e2);
-  Free(len_e3);
-  Free(len_e4);
-  Free(bl_init);
+  Free_Scalar_Dbl(len_e1);
+  Free_Scalar_Dbl(len_e2);
+  Free_Scalar_Dbl(len_e3);
+  Free_Scalar_Dbl(len_e4);
+  Free_Scalar_Dbl(l_init);
+  Free_Scalar_Dbl(var_e1);
+  Free_Scalar_Dbl(var_e2);
+  Free_Scalar_Dbl(var_e3);
+  Free_Scalar_Dbl(var_e4);
+  Free_Scalar_Dbl(v_init);
 
   return result;
 }
@@ -792,13 +771,12 @@ void Make_Target_Swap(t_tree *tree, t_edge *b_fcus, int swaptodo)
 {
   t_node *v1,*v2,*v3,*v4;
   phydbl lktodo;
-  phydbl l_infa, l_infb;
   phydbl lk_init, lk_temp;
   int i;
 
   if(swaptodo < 0)
     {
-      PhyML_Printf("\n== Err in file %s at line %d\n\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n== Err in file %s at line %d\n\n",__FILE__,__LINE__);
       Warn_And_Exit("");
     }
 
@@ -817,12 +795,12 @@ void Make_Target_Swap(t_tree *tree, t_edge *b_fcus, int swaptodo)
 
   if(v1->num < v2->num)
     {
-      PhyML_Printf("\n== Err in file %s at line %d\n\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n== Err in file %s at line %d\n\n",__FILE__,__LINE__);
       Warn_And_Exit("");
     }
   if(v3->num < v4->num)
     {
-      PhyML_Printf("\n. Err in file %s at line %d\n\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n. Err in file %s at line %d\n\n",__FILE__,__LINE__);
       Warn_And_Exit("");
     }
 
@@ -843,14 +821,14 @@ void Make_Target_Swap(t_tree *tree, t_edge *b_fcus, int swaptodo)
   MIXT_Set_Alias_Subpatt(YES,tree);
   Set_Both_Sides(YES,tree);
   lktodo = Update_Lk_At_Given_Edge(b_fcus,tree);
-
-  For(i,3)
+  
+  for(i=0;i<3;i++)
     if(b_fcus->left->v[i] != b_fcus->rght)
-      Update_P_Lk(tree,b_fcus->left->b[i],b_fcus->left);
-
-  For(i,3)
+      Update_Partial_Lk(tree,b_fcus->left->b[i],b_fcus->left);
+  
+  for(i=0;i<3;i++)
     if(b_fcus->rght->v[i] != b_fcus->left)
-      Update_P_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
+      Update_Partial_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
 
   MIXT_Set_Alias_Subpatt(NO,tree);
 
@@ -860,43 +838,32 @@ void Make_Target_Swap(t_tree *tree, t_edge *b_fcus, int swaptodo)
     {
       lktodo = lk_temp;
 
-      For(i,3)
-    if(b_fcus->left->v[i] != b_fcus->rght)
-      {
-        Update_P_Lk(tree,b_fcus->left->b[i],b_fcus->left);
-
-        l_infa  = 10.;
-        l_infb  = tree->mod->l_min/b_fcus->left->b[i]->l->v;
-        lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus->left->b[i],tree);
-      }
+      for(i=0;i<3;i++)
+        if(b_fcus->left->v[i] != b_fcus->rght)
+          {
+            Update_Partial_Lk(tree,b_fcus->left->b[i],b_fcus->left);
+            lk_temp = Br_Len_Opt(b_fcus->left->b[i],tree);
+          }
 
 
-      Update_P_Lk(tree,b_fcus,b_fcus->left);
+      Update_Partial_Lk(tree,b_fcus,b_fcus->left);
+      lk_temp = Br_Len_Opt(b_fcus,tree);
 
-      l_infa  = 10.;
-      l_infb  = tree->mod->l_min/b_fcus->l->v;
-      lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus,tree);
+      for(i=0;i<3;i++)
+        if(b_fcus->rght->v[i] != b_fcus->left)
+          {
+            Update_Partial_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
+            lk_temp = Br_Len_Opt(b_fcus->rght->b[i],tree);
+          }
 
-
-
-      For(i,3)
-    if(b_fcus->rght->v[i] != b_fcus->left)
-      {
-        Update_P_Lk(tree,b_fcus->rght->b[i],b_fcus->rght);
-
-        l_infa  = 10.;
-        l_infb  = tree->mod->l_min/b_fcus->rght->b[i]->l->v;
-        lk_temp = Br_Len_Brent(l_infb,l_infa,b_fcus->rght->b[i],tree);
-      }
-
-      Update_P_Lk(tree,b_fcus,b_fcus->rght);
+      Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
 
       if(lk_temp < lktodo - tree->mod->s_opt->min_diff_lk_local)
-    {
-      PhyML_Printf("\n. Edge %3d lk_temp = %f lktodo = %f\n",b_fcus->num,lk_temp,lktodo);
-      PhyML_Printf("\n. Err in file %s at line %d\n\n",__FILE__,__LINE__);
-      Warn_And_Exit("");
-    }
+        {
+          PhyML_Fprintf(stderr,"\n== Edge %3d lk_temp = %f lktodo = %f\n",b_fcus->num,lk_temp,lktodo);
+          PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n\n",__FILE__,__LINE__);
+          Warn_And_Exit("");
+        }
     }
   while(FABS(lk_temp-lktodo) > tree->mod->s_opt->min_diff_lk_global);
   //until no significative improvement is detected
@@ -913,10 +880,9 @@ void Make_Target_Swap(t_tree *tree, t_edge *b_fcus, int swaptodo)
 
   if(tree->c_lnL < lk_init - tree->mod->s_opt->min_diff_lk_global)
     {
-      PhyML_Printf("\n. [%3d] v1=%d v2=%d v3=%d v4=%d",
-         b_fcus->num,v1->num,v2->num,v3->num,v4->num);
-      PhyML_Printf("\n. tree->c_lnL = %f lk_init = %f\n",tree->c_lnL,lk_init);
-      PhyML_Printf("\n. Err in file %s at line %d\n\n",__FILE__,__LINE__);
+      PhyML_Fprintf(stderr,"\n== [%3d] v1=%d v2=%d v3=%d v4=%d",b_fcus->num,v1->num,v2->num,v3->num,v4->num);
+      PhyML_Fprintf(stderr,"\n== tree->c_lnL = %f lk_init = %f\n",tree->c_lnL,lk_init);
+      PhyML_Fprintf(stderr,"\n== Err. in file %s at line %d\n\n",__FILE__,__LINE__);
       Warn_And_Exit("");
     }
 }
@@ -1107,17 +1073,16 @@ phydbl Statistics_to_RELL(t_tree *tree)
   int i;
   int occurence=1000;
   phydbl nb=0.0;
-  phydbl res;
+  phydbl res,*pi;
   int site;
   phydbl lk0=0.0;
   phydbl lk1=0.0;
   phydbl lk2=0.0;
-  phydbl buff = -1.;
   int position = -1;
   t_tree *buff_tree;
 
   /*! 1000 times */
-  For(i,occurence)
+  for(i=0;i<occurence;i++)
     {
       lk0=0.0;
       lk1=0.0;
@@ -1127,19 +1092,19 @@ phydbl Statistics_to_RELL(t_tree *tree)
       buff_tree = tree;
       do
         {
+          pi = (phydbl *)mCalloc(tree->data->crunch_len,sizeof(phydbl));
+          for(site=0;site<tree->n_pattern;site++) pi[site] = tree->data->wght[site] / (phydbl)tree->data->init_len;
+
           For(site, tree->data->init_len)
             {
-              buff  = rand();
-              buff /= (RAND_MAX+1.);
-              buff *= tree->data->init_len;
-              position = (int)FLOOR(buff);
-
+              position = Sample_i_With_Proba_pi(pi,tree->n_pattern);
               lk0+=tree->log_lks_aLRT[0][position];
               lk1+=tree->log_lks_aLRT[1][position];
               lk2+=tree->log_lks_aLRT[2][position];
             }
           if (lk0>=lk1 && lk0>=lk2) nb++;
           tree = tree->next_mixt;
+          Free(pi);
         }
       while(tree);
       tree = buff_tree;
@@ -1169,22 +1134,22 @@ phydbl Statistics_To_SH(t_tree *tree)
   phydbl c0=0.0;
   phydbl c1=0.0;
   phydbl c2=0.0;
-  phydbl buff=-1.;
   int position=-1;
   phydbl delta_local=-1.;
   phydbl delta=0.0;
   t_tree *buff_tree;
+  phydbl *pi;
 
 
   /*! Compute the total log-lk of each NNI position */
   buff_tree = tree;
   do
     {
-      For(site, tree->data->init_len)
+      For(site, tree->n_pattern)
         {
-          c0+=tree->log_lks_aLRT[0][site];
-          c1+=tree->log_lks_aLRT[1][site];
-          c2+=tree->log_lks_aLRT[2][site];
+          c0+=tree->log_lks_aLRT[0][site] * tree->data->wght[site];
+          c1+=tree->log_lks_aLRT[1][site] * tree->data->wght[site];
+          c2+=tree->log_lks_aLRT[2][site] * tree->data->wght[site];
         }
       tree = tree->next_mixt;
     }
@@ -1227,7 +1192,7 @@ phydbl Statistics_To_SH(t_tree *tree)
     }
 
   /*! 1000 times */
-  For(i,occurence)
+  for(i=0;i<occurence;i++)
     {
       lk0=0.0;
       lk1=0.0;
@@ -1236,20 +1201,20 @@ phydbl Statistics_To_SH(t_tree *tree)
       buff_tree = tree;
       do
         {
+          pi = (phydbl *)mCalloc(tree->data->crunch_len,sizeof(phydbl));
+          for(site=0;site<tree->n_pattern;site++) pi[site] = tree->data->wght[site] / (phydbl)tree->data->init_len;
+
           /*! Shuffle the data */
           For(site, tree->data->init_len)
             {
-              buff  = rand();
-              buff /= (RAND_MAX+1.);
-              buff *= tree->data->init_len;
-              position = (int)FLOOR(buff);
-
+              position = Sample_i_With_Proba_pi(pi,tree->n_pattern);
               lk0+=tree->log_lks_aLRT[0][position];
               lk1+=tree->log_lks_aLRT[1][position];
               lk2+=tree->log_lks_aLRT[2][position];
             }
 
           tree = tree->next_mixt;
+          Free(pi);
         }
       while(tree);
       tree = buff_tree;
@@ -1320,9 +1285,9 @@ phydbl Statistics_To_SH(t_tree *tree)
 phydbl Update_Lk_At_Given_Edge_Excluding(t_edge *b_fcus, t_tree *tree, t_node *exclude)
 {
   if((!b_fcus->left->tax) && (exclude == NULL || exclude != b_fcus->left))
-    Update_P_Lk(tree,b_fcus,b_fcus->left);
+    Update_Partial_Lk(tree,b_fcus,b_fcus->left);
   if((!b_fcus->rght->tax) && (exclude == NULL || exclude != b_fcus->rght))
-    Update_P_Lk(tree,b_fcus,b_fcus->rght);
+    Update_Partial_Lk(tree,b_fcus,b_fcus->rght);
 
   tree->c_lnL = Lk(b_fcus,tree);
 
