@@ -1,7 +1,7 @@
 /*
 
-PhyML:  a program that  computes maximum likelihood phyLOGenies from
-DNA or AA homoLOGous sequences.
+PhyML:  a program that  computes maximum likelihood phylogenies from
+DNA or AA homologous sequences.
 
 Copyright (C) Stephane Guindon. Oct 2003 onward.
 
@@ -24,12 +24,14 @@ int Read_Command_Line(option *io, int argc, char **argv)
   int i;
   int writemode;
 
-  PhyML_Printf("\n. command-line: ");
-  For(i,argc) PhyML_Printf("%s ",argv[i]);
-
+  PhyML_Printf("\n. Command line: ");
+  for(i=0;i<argc;i++) PhyML_Printf("%s ",argv[i]);
+  PhyML_Printf("\n");
+  writemode = WRITE;
 
   if(argc == 1) Exit("\n. No argument was passed to the program. Please check the documentation. \n");
-
+  PhyML_Printf("",writemode);
+  
   struct option longopts[] =
     {
       {"n_rgrft",           required_argument,NULL,0},
@@ -123,6 +125,10 @@ int Read_Command_Line(option *io, int argc, char **argv)
       {"ancestral",           no_argument,NULL,76},
       {"anc",                 no_argument,NULL,76},
       {"coord_file",          required_argument,NULL,77},
+      {"json_trace",          no_argument,NULL,78},
+      {"weights",             required_argument,NULL,79},
+      {"tbe",                 no_argument,NULL,80},
+      {"leave_duplicates",    no_argument,NULL,81},
       {0,0,0,0}
     };
 
@@ -140,6 +146,27 @@ int Read_Command_Line(option *io, int argc, char **argv)
 
       switch(c)
 	{
+	case 81 :
+	  {
+	    io->leave_duplicates = YES;
+	    break;
+	  }
+	case 80 :
+	  {
+	    io->tbe_bootstrap = YES;
+	    break;
+	  }
+        case 79:
+          {
+            io->has_io_weights = YES;
+            strcpy(io->weight_file, optarg);
+            break;
+          }
+        case 78:
+          {
+	    io->print_json_trace = YES;
+	    break;            
+          }
 
         case 77:
           {
@@ -196,20 +223,33 @@ int Read_Command_Line(option *io, int argc, char **argv)
           }
 	case 73:
 	  {
-#ifndef INVITEE
-            Free_Optimiz(io->mod->s_opt);
-            M4_Free_M4_Model(io->mod->m4mod);
-            Free_Model_Basic(io->mod);
-            Free_Input(io);
-            PhyML_XML(optarg);
-            return 0;
-#else
+#ifdef INVITEE
+            
             Free_Optimiz(io->mod->s_opt);
             M4_Free_M4_Model(io->mod->m4mod);
             Free_Model_Basic(io->mod);
             Free_Input(io);
             PhyTime_XML(optarg);
             return 0;
+
+#elif defined(PHYML)
+           
+            Free_Optimiz(io->mod->s_opt);
+            M4_Free_M4_Model(io->mod->m4mod);
+            Free_Model_Basic(io->mod);
+            Free_Input(io);
+            io = PhyML_XML(optarg);
+            Free(io);
+            return 0;
+
+#elif defined(PHYTIME)
+
+            Free_Optimiz(io->mod->s_opt);
+            M4_Free_M4_Model(io->mod->m4mod);
+            Free_Model_Basic(io->mod);
+            Free_Input(io);
+            DATE_XML(optarg);
+            return 0;
 #endif
             break;
 	  }
@@ -300,8 +340,8 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	    i = 0;
 	    while(optarg[i++]) s[i]=tolower(optarg[i]);
 	    if(!strcmp(optarg,"gbd")) io->rates->model               = THORNE;
-	    else if(!strcmp(optarg,"gbs")) io->rates->model          = GUINDON;
-	    else if(!strcmp(optarg,"gamma")) io->rates->model        = GAMMA;
+	    else if(!strcmp(optarg,"geom")) io->rates->model         = GUINDON;
+	    else if(!strcmp(optarg,"lognorm")) io->rates->model      = LOGNORMAL;
 	    else if(!strcmp(optarg,"clock")) io->rates->model        = STRICTCLOCK;
 	    else if(!strcmp(optarg,"strictclock")) io->rates->model  = STRICTCLOCK;
 	    else if(!strcmp(optarg,"strict_clock")) io->rates->model = STRICTCLOCK;
@@ -333,7 +373,6 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  }
 	case 59:
 	  {
-	    io->mcmc->use_data = NO;
 	    break;
 	  }
 	case 58:
@@ -442,9 +481,12 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  }
 	case 46 : 
 	  {
-	    io->append_run_ID = YES;
-	    strcpy(io->run_id_string,optarg);
-	    break;
+            if(strlen(optarg) > 0)
+              {
+                io->append_run_ID = YES;
+                strcpy(io->run_id_string,optarg);
+              }
+            break;
 	  }
 	case 45 : 
 	  {
@@ -487,7 +529,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  }
 	case 40 :
 	  {
-	    writemode = 2;
+	    writemode = APPEND;
 	    break;
 	  }
 	case 39 :
@@ -508,9 +550,9 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  }
 	case 36 :
 	  {
-	    #ifndef PHYML
+#ifndef PHYML
             open_ps_file = 1;
-            #endif
+#endif
 	    break;
 	  }
 	case 35 :
@@ -681,7 +723,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  {
 	    io->mod->s_opt->random_input_tree = 1;
 	    io->mod->s_opt->n_rand_starts = atoi(optarg);
-	    if(io->mod->s_opt->n_rand_starts < 1) Exit("\n== Number of random starting trees must be > 0.\n\n");
+	    if(io->mod->s_opt->n_rand_starts < 1) Exit("\n. Number of random starting trees must be > 0.\n\n");
 	  }
 	case 's':case 6:
 	  {
@@ -692,11 +734,13 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	      }
 	    else if((!strcmp(optarg,"nni")) || (!strcmp(optarg,"NNI")))
 	      {
+                PhyML_Printf("\n. The NNI option is deprecated. PhyML now uses a mix of SPRs and NNIs.");
 		io->mod->s_opt->topo_search         = NNI_MOVE;
 		io->mod->s_opt->random_input_tree   = 0;
 	      }
 	    else if((!strcmp(optarg,"best")) || (!strcmp(optarg,"BEST")))
 	      {
+                PhyML_Printf("\n. The BEST option is deprecated. PhyML now uses a mix of SPRs and NNIs.");
 		io->mod->s_opt->topo_search = BEST_OF_NNI_AND_SPR;
 		io->mod->s_opt->greedy      = (io->mod->s_opt->steph_spr)?(0):(1);
 	      }
@@ -835,83 +879,105 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	    else if(strcmp(optarg, "DAYHOFF") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = DAYHOFF;
 	      }
 	    else if(strcmp (optarg, "JTT") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = JTT;
 	      }
 	    else if(strcmp(optarg, "MTREV") == 0)
 	      {
 		io->datatype             = AA;
+		io->mod->ns              = 20;
 		io->mod->whichmodel      = MTREV;
 	      }
 	    else if(strcmp (optarg, "LG") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = LG;
 	      }
 	    else if(strcmp (optarg, "WAG") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = WAG;
 	      }
 	    else if(strcmp(optarg, "DCMUT") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = DCMUT;
 	      }
 	    else if(strcmp (optarg, "RTREV") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = RTREV;
 	      }
 	    else if(strcmp(optarg, "CPREV") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = CPREV;
 	      }
 	    else if(strcmp(optarg, "VT") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = VT;
 	      }
 	    else if(strcmp(optarg, "BLOSUM62") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = BLOSUM62;
 	      }
 	    else if(strcmp(optarg, "MTMAM") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = MTMAM;
 	      }
 	    else if (strcmp(optarg,"MTART") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = MTART;
 	      }
 	    else if (strcmp(optarg,"HIVW") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = HIVW;
 	      }
 	    else if(strcmp(optarg, "HIVB") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = HIVB;
 	      }
 	    else if(strcmp(optarg, "AB") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = AB;
 	      }
 	    else if (strcmp(optarg, "CUSTOM") == 0)
 	      {
 		io->datatype              = AA;
+		io->mod->ns               = 20;
 		io->mod->whichmodel       = CUSTOMAA;
 	      }
+	    else if(strcmp(optarg, "FLU") == 0)
+	      {
+		io->datatype              = AA;
+		io->mod->ns               = 20;
+		io->mod->whichmodel       = FLU;
+	      }
 	    else
 	      {
 		PhyML_Printf("\n. The model name is incorrect. Please see the documentation.\n");
@@ -1039,40 +1105,38 @@ int Read_Command_Line(option *io, int argc, char **argv)
 		phydbl sum;
 		double val1,val2,val3,val4;
 		
-		io->mod->s_opt->opt_state_freq  = 0;
-		io->mod->s_opt->user_state_freq = 1;
-		
-		/* 		sscanf(optarg,"%lf,%lf,%lf,%lf", */
-		/* 		       io->mod->user_b_freq, */
-		/* 		       io->mod->user_b_freq+1, */
-		/* 		       io->mod->user_b_freq+2, */
-		/* 		       io->mod->user_b_freq+3); */
+                io->mod->e_frq = (t_efrq *)Make_Efrq(4);
+                Init_Efrq(NULL,io->mod->e_frq);
+
+		io->mod->s_opt->opt_state_freq  = NO;
+                io->mod->e_frq->user_state_freq = YES;
+
 		sscanf(optarg,"%lf,%lf,%lf,%lf",&val1,&val2,&val3,&val4);
-		io->mod->user_b_freq->v[0] = (phydbl)val1;
-		io->mod->user_b_freq->v[1] = (phydbl)val2;
-		io->mod->user_b_freq->v[2] = (phydbl)val3;
-		io->mod->user_b_freq->v[3] = (phydbl)val4;
+		io->mod->e_frq->user_b_freq->v[0] = (phydbl)val1;
+		io->mod->e_frq->user_b_freq->v[1] = (phydbl)val2;
+		io->mod->e_frq->user_b_freq->v[2] = (phydbl)val3;
+		io->mod->e_frq->user_b_freq->v[3] = (phydbl)val4;
 		
 		sum =
-		  (io->mod->user_b_freq->v[0] +
-		   io->mod->user_b_freq->v[1] +
-		   io->mod->user_b_freq->v[2] +
-		   io->mod->user_b_freq->v[3]);
+		  (io->mod->e_frq->user_b_freq->v[0] +
+		   io->mod->e_frq->user_b_freq->v[1] +
+		   io->mod->e_frq->user_b_freq->v[2] +
+		   io->mod->e_frq->user_b_freq->v[3]);
 		
-		io->mod->user_b_freq->v[0] /= sum;
-		io->mod->user_b_freq->v[1] /= sum;
-		io->mod->user_b_freq->v[2] /= sum;
-		io->mod->user_b_freq->v[3] /= sum;
+		io->mod->e_frq->user_b_freq->v[0] /= sum;
+		io->mod->e_frq->user_b_freq->v[1] /= sum;
+		io->mod->e_frq->user_b_freq->v[2] /= sum;
+		io->mod->e_frq->user_b_freq->v[3] /= sum;
 		
-		
-		if(io->mod->user_b_freq->v[0] < .0 ||
-		   io->mod->user_b_freq->v[1] < .0 ||
-		   io->mod->user_b_freq->v[2] < .0 ||
-		   io->mod->user_b_freq->v[3] < .0 ||
-		   io->mod->user_b_freq->v[0] > 1. ||
-		   io->mod->user_b_freq->v[1] > 1. ||
-		   io->mod->user_b_freq->v[2] > 1. ||
-		   io->mod->user_b_freq->v[3] > 1.)
+                
+		if(io->mod->e_frq->user_b_freq->v[0] < .0 ||
+		   io->mod->e_frq->user_b_freq->v[1] < .0 ||
+		   io->mod->e_frq->user_b_freq->v[2] < .0 ||
+		   io->mod->e_frq->user_b_freq->v[3] < .0 ||
+		   io->mod->e_frq->user_b_freq->v[0] > 1. ||
+		   io->mod->e_frq->user_b_freq->v[1] > 1. ||
+		   io->mod->e_frq->user_b_freq->v[2] > 1. ||
+		   io->mod->e_frq->user_b_freq->v[3] > 1.)
 		  {
 		    Warn_And_Exit("\n. Invalid base frequencies.\n");
 		  }
@@ -1314,7 +1378,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	    else
 	      {
 		char choix;
-		PhyML_Printf ("\n. The optimization parameter must be 'tlr' or 'tl' or 'lr' or 'l' or 'r' or ''.");
+		PhyML_Printf ("\n. The optimization parameter must be 'tlr' or 'tl' or 'lr' or 'l' or 'r' or 'n'.");
 		PhyML_Printf("\n. Type any key to exit.\n");
 		if(!scanf("%c",&choix)) Exit("\n");
 		Exit("\n");
@@ -1378,29 +1442,26 @@ int Read_Command_Line(option *io, int argc, char **argv)
      (io->mod->s_opt->topo_search == NNI_MOVE) && 
      (io->mod->s_opt->random_input_tree))
     {
-      Warn_And_Exit("\n== The random starting tree option is only compatible with SPR based search options.\n"); 
+      Warn_And_Exit("\n. The random starting tree option is only compatible with SPR based search options.\n"); 
     }
   
   if ((io->datatype == NT) && (io->mod->whichmodel > 10))
     {
       char choix;
-      PhyML_Printf("\n== Err.: model incompatible with the data type. Please use JC69, K80, F81, HKY, F84, TN93 or GTR\n");
-      PhyML_Printf("\n== Type any key to exit.\n");
+      PhyML_Printf("\n. Err.: model incompatible with the data type. Please use JC69, K80, F81, HKY, F84, TN93 or GTR\n");
+      PhyML_Printf("\n. Type any key to exit.\n");
       if(!scanf("%c",&choix)) Exit("\n");
       Warn_And_Exit("\n");
     }
   else if ((io->datatype == AA) && (io->mod->whichmodel < 11))
     {
       char choix;
-      PhyML_Printf("\n== Err.: model incompatible with the data type. Please use LG, Dayhoff, JTT, MtREV, WAG, DCMut, RtREV, CpREV, VT, Blosum62, MtMam, MtArt, HIVw, HIVb or AB.\n");
-      PhyML_Printf("\n== Type any key to exit.\n");
+      PhyML_Printf("\n. Err.: model incompatible with the data type. Please use LG, Dayhoff, JTT, MtREV, WAG, DCMut, RtREV, CpREV, VT, Blosum62, MtMam, MtArt, HIVw, HIVb or AB.\n");
+      PhyML_Printf("\n. Type any key to exit.\n");
       if(!scanf("%c",&choix)) Exit("\n");
       Exit("\n");
     }
   
-
-  
-
   if(io->mod->use_m4mod == NO)
     {
       io->mod->s_opt->opt_cov_delta      = 0;
@@ -1415,60 +1476,81 @@ int Read_Command_Line(option *io, int argc, char **argv)
       io->mod->m4mod->use_cov_free       = 1;
     }
   
-  if(io->print_site_lnl)
+  if(io->print_site_lnl && io->fp_in_align != NULL)
     {
       strcpy(io->out_lk_file,io->in_align_file);
       strcat(io->out_lk_file, "_phyml_lk");
       if(io->append_run_ID) { strcat(io->out_lk_file,"_"); strcat(io->out_lk_file,io->run_id_string); }
+      strcat(io->out_lk_file, ".txt");
       io->fp_out_lk = Openfile(io->out_lk_file,1);
     }
   
-  if(io->print_trace)
+  if(io->print_trace && io->fp_in_align != NULL)
     {
       strcpy(io->out_trace_file,io->in_align_file);
       strcat(io->out_trace_file,"_phyml_trace");
       if(io->append_run_ID) { strcat(io->out_trace_file,"_"); strcat(io->out_trace_file,io->run_id_string); }
-      io->fp_out_trace = Openfile(io->out_trace_file,1);
+      strcat(io->out_trace_file,".txt");
+      io->fp_out_trace = Openfile(io->out_trace_file,WRITE);
     }
+
+  if(io->print_json_trace && io->fp_in_align != NULL)
+    {
+      strcpy(io->out_json_trace_file,io->in_align_file);
+      strcat(io->out_json_trace_file,"_phyml_trace");
+      if(io->append_run_ID) { strcat(io->out_json_trace_file,"_"); strcat(io->out_json_trace_file,io->run_id_string); }
+      strcat(io->out_json_trace_file,".json");
+      io->fp_out_json_trace = Openfile(io->out_json_trace_file,READWRITE);
+    }
+
   
-  if(io->mod->s_opt->random_input_tree)
+  if(io->mod->s_opt->random_input_tree && io->fp_in_align != NULL)
     {
       strcpy(io->out_trees_file,io->in_align_file);
       strcat(io->out_trees_file,"_phyml_rand_trees");
       if(io->append_run_ID) { strcat(io->out_trees_file,"_"); strcat(io->out_trees_file,io->run_id_string); }
+      strcat(io->out_trees_file,".txt");
       io->fp_out_trees = Openfile(io->out_trees_file,1);
     }
   
-  if((io->print_boot_trees) && (io->mod->bootstrap > 0))
+  if((io->print_boot_trees) && (io->mod->bootstrap > 0) && (io->fp_in_align != NULL))
     {
       strcpy(io->out_boot_tree_file,io->in_align_file);
       strcat(io->out_boot_tree_file,"_phyml_boot_trees");
       if(io->append_run_ID) { strcat(io->out_boot_tree_file,"_"); strcat(io->out_boot_tree_file,io->run_id_string); }
+      strcat(io->out_boot_tree_file,".txt");
       io->fp_out_boot_tree = Openfile(io->out_boot_tree_file,1);
       
       strcpy(io->out_boot_stats_file,io->in_align_file);
       strcat(io->out_boot_stats_file,"_phyml_boot_stats");
       if(io->append_run_ID) { strcat(io->out_boot_stats_file,"_"); strcat(io->out_boot_stats_file,io->run_id_string); }
+      strcat(io->out_boot_stats_file,".txt");
       io->fp_out_boot_stats = Openfile(io->out_boot_stats_file,1);
     }
   
-  if(io->append_run_ID)
+  if(io->append_run_ID && io->fp_in_align != NULL)
     {
       strcat(io->out_tree_file,"_");
       strcat(io->out_stats_file,"_");
       strcat(io->out_tree_file,io->run_id_string);
       strcat(io->out_stats_file,io->run_id_string);
     }
-  
+
+  if(io->fp_in_align != NULL)
+    {
+      strcat(io->out_tree_file,".txt");
+      strcat(io->out_stats_file,".txt");
+    }
+
   if(io->mod->ras->n_catg == 1) io->mod->s_opt->opt_alpha = 0;
   
   if(!io->mod->s_opt->opt_subst_param)
     {
-      io->mod->s_opt->opt_alpha  = 0;
-      io->mod->s_opt->opt_kappa  = 0;
-      io->mod->s_opt->opt_lambda = 0;
-      io->mod->s_opt->opt_pinvar = 0;
-      io->mod->s_opt->opt_rr     = 0;	
+      io->mod->s_opt->opt_alpha  = NO;
+      io->mod->s_opt->opt_kappa  = NO;
+      io->mod->s_opt->opt_lambda = NO;
+      io->mod->s_opt->opt_pinvar = NO;
+      io->mod->s_opt->opt_rr     = NO;	
     }
   
   if(io->mod->whichmodel != K80 && 
@@ -1481,27 +1563,30 @@ int Read_Command_Line(option *io, int argc, char **argv)
   
   if(io->datatype == AA && io->mod->whichmodel == CUSTOMAA && !io->mod->fp_aa_rate_mat)
     {
-      PhyML_Printf("\n== Custom model option with amino-acid requires you to specify a rate matrix file through the '--aa_rate_file' option.\n");
+      PhyML_Printf("\n. Custom model option with amino-acid requires you to specify a rate matrix file through the '--aa_rate_file' option.\n");
       Exit("\n");
     }
   
 #if !defined(PHYTIME) 
   // Make sure you don't erase the input file...
   if(!strcmp(io->out_tree_file,io->in_align_file) ||
-     !strcmp(io->out_stats_file,io->in_align_file)) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);    
-
-  writemode = WRITE;
-
-  io->fp_out_tree  = Openfile(io->out_tree_file,writemode);
-  io->fp_out_stats = Openfile(io->out_stats_file,writemode);
+     !strcmp(io->out_stats_file,io->in_align_file)) 
+    {
+      PhyML_Fprintf(stderr,"\n. The alignment file '%s' does not seem to exist...",io->in_align_file);
+      Generic_Exit(__FILE__,__LINE__,__FUNCTION__);    
+    }
+  
+  if(io->fp_in_align != NULL)
+    {
+      io->fp_out_tree  = Openfile(io->out_tree_file,writemode);
+      io->fp_out_stats = Openfile(io->out_stats_file,writemode);
+    }
 #endif
   
 
 #if defined(PHYREX)
-  io->fp_out_summary = Openfile(io->out_summary_file,writemode);
+  if(io->fp_in_align != NULL) io->fp_out_summary = Openfile(io->out_summary_file,writemode);
 #endif
-
-  writemode++; // just to silence a warning message at compilation
   
   if(io->mod->whichmodel == GTR) 
     {
@@ -1509,7 +1594,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
       io->mod->s_opt->opt_rr = 1;
     }
   
-
+  
   return 1;
 }
 
