@@ -1,6 +1,6 @@
 /*
 
-PHYML :  a program that  computes maximum likelihood  phyLOGenies from
+PHYML :  a program that  computes maximum likelihood  phylogenies from
 DNA or AA homologous sequences
 
 Copyright (C) Stephane Guindon. Oct 2003 onward
@@ -103,6 +103,8 @@ void Free_Node(t_node *n)
   Free(n->score);
   Free(n->s_ingrp);
   Free(n->s_outgrp);
+  Free(n->cal);
+
   if(n->c_seq_anc != NULL) 
     {
       Free(n->c_seq_anc->state);
@@ -125,7 +127,7 @@ void Free_Mat(matrix *mat)
 {
   int i;
 
-  For(i,mat->n_otu)
+  for(i=0;i<mat->n_otu;i++)
     {
       Free(mat->P[i]);
       Free(mat->Q[i]);
@@ -152,9 +154,9 @@ void Free_Partial_Lk(phydbl *p_lk, int len, int n_catg)
   Free(p_lk);
 
 /*   int i,j; */
-/*   For(i,len) */
+/*   for(i=0;i<len;i++) */
 /*     { */
-/*       For(j,n_catg) Free((*p_lk)[i][j]); */
+/*       for(j=0;j<n_catg;j++) Free((*p_lk)[i][j]); */
 /*       Free((*p_lk)[i]); */
 /*     } */
 /*   Free((*p_lk)); */
@@ -212,7 +214,7 @@ void Free_Bip(t_tree *tree)
       For(i,2*tree->n_otu-2)
     {
       Free(tree->a_nodes[i]->bip_size);
-      For(j,3) Free(tree->a_nodes[i]->bip_node[j]);
+      for(j=0;j<3;j++) Free(tree->a_nodes[i]->bip_node[j]);
       Free(tree->a_nodes[i]->bip_node);
     }
     }
@@ -222,16 +224,17 @@ void Free_Bip(t_tree *tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-void Free_Cseq(calign *data)
+void Free_Calign(calign *data)
 {
   int i;
 
+  if(data->io_wght) Free_Scalar_Dbl(data->io_wght);
   Free(data->invar);
   Free(data->wght);
   Free(data->ambigu);
-  Free(data->b_frq);
+  Free(data->obs_state_frq);
   Free(data->sitepatt);
-  For(i,data->n_otu)
+  for(i=0;i<data->n_otu;i++)
     {
       Free(data->c_seq[i]->name);
       if(data->c_seq[i]->state)
@@ -242,7 +245,23 @@ void Free_Cseq(calign *data)
         }
       Free(data->c_seq[i]);
     }
-  Free(data->c_seq);
+
+
+  for(i=0;i<data->n_rm;i++)
+    {
+      Free(data->c_seq_rm[i]->name);
+      if(data->c_seq_rm[i]->state)
+        {
+          Free(data->c_seq_rm[i]->state);
+          Free(data->c_seq_rm[i]->d_state);
+          if(data->c_seq_rm[i]->is_ambigu) Free(data->c_seq_rm[i]->is_ambigu);
+        }
+      Free(data->c_seq_rm[i]);
+    }
+  
+  if(data->c_seq_rm != NULL) Free(data->c_seq_rm);
+  if(data->c_seq != NULL) Free(data->c_seq);
+
   Free(data);
 }
 
@@ -252,7 +271,7 @@ void Free_Cseq(calign *data)
 void Free_Seq(align **d, int n_otu)
 {
   int i;
-  For(i,n_otu)
+  for(i=0;i<n_otu;i++)
     {
       Free(d[i]->name);
       Free(d[i]->state);
@@ -270,7 +289,7 @@ void Free_Seq(align **d, int n_otu)
 
 void Free_All(align **d, calign *cdata, t_tree *tree)
 {
-  Free_Cseq(cdata);
+  Free_Calign(cdata);
   Free_Seq(d,tree->n_otu);
   Free_Tree(tree);
 }
@@ -285,7 +304,7 @@ void Free_SubTree(t_edge *b_fcus, t_node *a, t_node *d, t_tree *tree)
   if(d->tax) return;
   else
     {
-      For(i,3)
+      for(i=0;i<3;i++)
     {
       if(d->v[i] != a)
         {
@@ -318,20 +337,16 @@ void Free_Tree_Pars(t_tree *mixt_tree)
     {
       Free(tree->step_mat);
       Free(tree->site_pars);
-
+      
       For(i,2*tree->n_otu-3) 
         {
-          /* printf("\n. FREE b->num: %d %p %p",tree->a_edges[i]->num,tree->a_edges[i]->pars_l,tree->a_edges[i]->pars_r); */
           Free_Edge_Pars(tree->a_edges[i]);
         }
 
       if(tree->n_root)
         {
-          /* printf("\n. FREE b1->num: %d %p %p",tree->n_root->b[1]->num,tree->n_root->b[1]->pars_l,tree->n_root->b[1]->pars_r); */
           Free_Edge_Pars_Left(tree->n_root->b[1]);
           Free_Edge_Pars_Left(tree->n_root->b[2]);
-          /* if(tree->n_root->b[1]->pars_r) Free_Edge_Pars_Rght(tree->n_root->b[1]); */
-          /* if(tree->n_root->b[2]->pars_r) Free_Edge_Pars_Rght(tree->n_root->b[2]); */
         }
       else
         {
@@ -378,7 +393,6 @@ void Free_Edge_Pars(t_edge *b)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 void Free_Tree_Lk(t_tree *mixt_tree)
 {
   int i;
@@ -392,31 +406,59 @@ void Free_Tree_Lk(t_tree *mixt_tree)
       Free(tree->old_site_lk);
       Free(tree->site_lk_cat);
       Free(tree->fact_sum_scale);
-
-      For(i,3) Free(tree->log_lks_aLRT[i]);
+      Free(tree->eigen_lr_left);
+      Free(tree->eigen_lr_rght);
+      Free(tree->dot_prod);
+      Free(tree->expl);
+      
+      for(i=0;i<3;i++) Free(tree->log_lks_aLRT[i]);
       Free(tree->log_lks_aLRT);
 
+      Free(tree->div_post_pred_extra_0);
+      Free(tree->sum_scale_cat_extra_0);
+      Free(tree->sum_scale_extra_0);
+      Free(tree->patt_id_extra_0);
+      Free(tree->p_lk_extra_0);
+      Free(tree->p_lk_tip_extra_0);
+
+      Free(tree->div_post_pred_extra_1);
+      Free(tree->sum_scale_cat_extra_1);
+      Free(tree->sum_scale_extra_1);
+      Free(tree->patt_id_extra_1);
+      Free(tree->p_lk_extra_1);
+      Free(tree->p_lk_tip_extra_1);
+      
       Free(tree->unscaled_site_lk_cat);
-
-      For(i,2*tree->n_otu-3) Free_Edge_Lk(tree->a_edges[i]);
-
-      if(tree->n_root)
+      
+      For(i,2*tree->n_otu-1) Free_NNI(tree->a_edges[i]->nni);
+  
+      if(tree->is_mixt_tree == NO)
         {
-          Free(tree->n_root->b[1]->nni);
-          Free(tree->n_root->b[2]->nni);
-          Free(tree->n_root->b[1]->Pij_rr);
-          Free(tree->n_root->b[2]->Pij_rr);
-          Free_Edge_Lk_Left(tree->n_root->b[1]);
-          Free_Edge_Lk_Left(tree->n_root->b[2]);
-          /* if(tree->n_root->b[1]->p_lk_rght) Free_Edge_Lk_Rght(tree->n_root->b[1]); */
-          /* if(tree->n_root->b[2]->p_lk_rght) Free_Edge_Lk_Rght(tree->n_root->b[2]); */
-        }
-      else
-        {
-          Free_Edge_Lk(tree->a_edges[2*tree->n_otu-3]);
-          Free_Edge_Lk(tree->a_edges[2*tree->n_otu-2]);
+          For(i,2*tree->n_otu-3) Free_Edge_Lk(tree->a_edges[i]);
+          For(i,2*tree->n_otu-3) Free_Edge_Loc(tree->a_edges[i]);
+          
+          if(tree->n_root != NULL)
+            {
+              Free(tree->n_root->b[1]->Pij_rr);
+              Free(tree->n_root->b[2]->Pij_rr);
+              Free(tree->n_root->b[1]->tPij_rr);
+              Free(tree->n_root->b[2]->tPij_rr);
+              Free_Edge_Lk_Left(tree->n_root->b[1]);
+              Free_Edge_Lk_Left(tree->n_root->b[2]);
+              Free_Edge_Loc_Left(tree->n_root->b[1]);
+              Free_Edge_Loc_Left(tree->n_root->b[2]);
+            }
+          else
+            {
+              Free_Edge_Lk(tree->a_edges[2*tree->n_otu-3]);
+              Free_Edge_Lk(tree->a_edges[2*tree->n_otu-2]);
+              Free_Edge_Loc(tree->a_edges[2*tree->n_otu-3]);
+              Free_Edge_Loc(tree->a_edges[2*tree->n_otu-2]);
+            }
         }
+      
       tree = tree->next;
+
     }
   while(tree);
 
@@ -434,6 +476,26 @@ void Free_Node_Lk(t_node *n)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+void Free_Extra_Edge_Lk(t_tree *tree)
+{
+  if(tree->div_post_pred_extra_0) Free(tree->div_post_pred_extra_0);
+  if(tree->sum_scale_cat_extra_0) Free(tree->sum_scale_cat_extra_0);
+  if(tree->sum_scale_extra_0) Free(tree->sum_scale_extra_0);
+  if(tree->p_lk_extra_0) Free(tree->p_lk_extra_0);
+  if(tree->p_lk_tip_extra_0) Free(tree->p_lk_tip_extra_0);
+  if(tree->patt_id_extra_0) Free(tree->patt_id_extra_0);
+
+  if(tree->div_post_pred_extra_1) Free(tree->div_post_pred_extra_1);
+  if(tree->sum_scale_cat_extra_1) Free(tree->sum_scale_cat_extra_1);
+  if(tree->sum_scale_extra_1) Free(tree->sum_scale_extra_1);
+  if(tree->p_lk_extra_1) Free(tree->p_lk_extra_1);
+  if(tree->p_lk_tip_extra_1) Free(tree->p_lk_tip_extra_1);
+  if(tree->patt_id_extra_1) Free(tree->patt_id_extra_1);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 void Free_Edge_Lk_Rght(t_edge *b)
 {
   Free(b->div_post_pred_rght);
@@ -444,12 +506,9 @@ void Free_Edge_Lk_Rght(t_edge *b)
       if(b->sum_scale_rght) Free(b->sum_scale_rght);
     }
 
-  if(b->p_lk_tip_r) Free(b->p_lk_tip_r);
-
-  Free(b->sum_scale_rght_cat);
-  Free(b->patt_id_rght);
-  Free(b->p_lk_loc_rght);
-
+  if(b->p_lk_tip_r)         Free(b->p_lk_tip_r);
+  if(b->sum_scale_rght_cat) Free(b->sum_scale_rght_cat);
+  if(b->patt_id_rght)       Free(b->patt_id_rght);
 }
 
 //////////////////////////////////////////////////////////////
@@ -466,12 +525,9 @@ void Free_Edge_Lk_Left(t_edge *b)
       if(b->sum_scale_left) Free(b->sum_scale_left);
     }
 
-  if(b->p_lk_tip_l) Free(b->p_lk_tip_l);
-
-  Free(b->sum_scale_left_cat);
-  Free(b->patt_id_left);
-  Free(b->p_lk_loc_left);
-
+  if(b->p_lk_tip_l)         Free(b->p_lk_tip_l);
+  if(b->sum_scale_left_cat) Free(b->sum_scale_left_cat);
+  if(b->patt_id_left)       Free(b->patt_id_left);
 }
 
 //////////////////////////////////////////////////////////////
@@ -479,8 +535,9 @@ void Free_Edge_Lk_Left(t_edge *b)
 
 void Free_Edge_Lk(t_edge *b)
 {
-  Free(b->nni);
+  Free(b->tPij_rr);
   Free(b->Pij_rr);
+  
   Free_Edge_Lk_Left(b);
   Free_Edge_Lk_Rght(b);
 }
@@ -488,6 +545,31 @@ void Free_Edge_Lk(t_edge *b)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+void Free_Edge_Loc_Rght(t_edge *b)
+{
+  if(b->p_lk_loc_rght) Free(b->p_lk_loc_rght);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Free_Edge_Loc_Left(t_edge *b)
+{
+  if(b->p_lk_loc_left) Free(b->p_lk_loc_left);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Free_Edge_Loc(t_edge *b)
+{
+  Free_Edge_Loc_Left(b);
+  Free_Edge_Loc_Rght(b);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 void Free_Model_Complete(t_mod *mixt_mod)
 {
   Free_Eigen(mixt_mod->eigen);
@@ -501,19 +583,60 @@ void Free_Model_Complete(t_mod *mixt_mod)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+void Free_Rmat_Weights(t_mod *mixt_mod)
+{
+  t_mod *mod;
+
+  mod = mixt_mod;
+
+  do
+    {
+      Free(mod->r_mat_weight);
+      mod = mod->next_mixt;
+    }
+  while(mod);
+
+  if(mixt_mod->next) Free_Scalar_Dbl(mixt_mod->next->r_mat_weight);
+
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Free_Efrq_Weights(t_mod *mixt_mod)
+{
+  t_mod *mod;
+
+  mod = mixt_mod;
+
+  do
+    {
+      Free(mod->e_frq_weight);
+      mod = mod->next_mixt;
+    }
+  while(mod);
+
+  if(mixt_mod->next) Free_Scalar_Dbl(mixt_mod->next->e_frq_weight);
+
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 void Free_Model_Basic(t_mod *mixt_mod)
 {
   t_mod *mod;
 
   Free_RAS(mixt_mod->ras);
-  Free_Vect_Dbl(mixt_mod->user_b_freq);
   Free_Scalar_Dbl(mixt_mod->mr);
   Free_Scalar_Dbl(mixt_mod->kappa);
   Free_Scalar_Dbl(mixt_mod->lambda);
   Free_Scalar_Dbl(mixt_mod->br_len_mult);
   Free_Scalar_Dbl(mixt_mod->br_len_mult_unscaled);
-  Free_Scalar_Dbl(mixt_mod->e_frq_weight);
-  Free_Scalar_Dbl(mixt_mod->r_mat_weight);
+
+  Free_Rmat_Weights(mixt_mod);
+  Free_Efrq_Weights(mixt_mod);
+
   Free_String(mixt_mod->modelname);
   Free_String(mixt_mod->custom_mod_string);
   Free_String(mixt_mod->aa_rate_mat_file);
@@ -543,6 +666,29 @@ void Free_Vect_Dbl(vect_dbl *v)
 {
   vect_dbl *next;
 
+  assert(v);
+
+  next = v->next;
+  do
+    {
+      Free(v->v);
+      Free(v);
+
+      v = next;
+      if(v) next = v->next;
+    }
+  while(v);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Free_Vect_Int(vect_int *v)
+{
+  vect_int *next;
+
+  assert(v);
+
   next = v->next;
   do
     {
@@ -561,6 +707,8 @@ void Free_Vect_Dbl(vect_dbl *v)
 void Free_Scalar_Dbl(scalar_dbl *v)
 {
   scalar_dbl *next;
+  
+  /* assert(v); */
 
   next = v->next;
   do
@@ -575,6 +723,50 @@ void Free_Scalar_Dbl(scalar_dbl *v)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+void Free_Scalar_Int(scalar_int *v)
+{
+  scalar_int *next;
+  
+  assert(v);
+
+  next = v->next;
+  do
+    {
+      Free(v);
+      v = next;
+      if(v) next = v->next;
+    }
+  while(v);
+}
+
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Free_Linked_List(t_ll *t)
+{
+  t_ll *next,*ll;
+
+  if(t == NULL) return;
+
+  ll = t->head;
+  next = ll->next;
+  do
+    {
+      /* t_node *n = ll->v; */
+      /* printf("\n. free node %d",n?n->num:-1); */
+      /* printf(" ll: %p",ll); */
+      /* printf(" hd: %p",ll?ll->head:NULL); fflush(NULL); */
+      Free(ll);
+      ll = next;
+      if(ll) next = ll->next;
+    }
+  while(ll);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 void Free_String(t_string *ts)
 {
   t_string *next;
@@ -609,6 +801,9 @@ void Free_Efrq(t_efrq *e_frq)
   Free(e_frq->pi_unscaled->v);
   Free(e_frq->pi_unscaled);
 
+  Free(e_frq->user_b_freq->v);
+  Free(e_frq->user_b_freq);
+
   if(e_frq->next) Free_Efrq(e_frq->next);
 
   Free(e_frq);
@@ -688,10 +883,18 @@ void Free_Model(t_mod *mod)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 void Free(void *p)
 {
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
   free(p);
+#else
+  _aligned_free(p);
+#endif
+#else  
+  free(p);
+#endif
+
   p = NULL;
 }
 
@@ -717,19 +920,21 @@ void Free_Input(option *io)
       Free(io->out_boot_tree_file);
       Free(io->out_boot_stats_file);
       Free(io->out_stats_file);
+      Free(io->weight_file);
       Free(io->out_lk_file);
       Free(io->out_summary_file);
       Free(io->out_ps_file);
       Free(io->out_trace_file);
+      Free(io->out_json_trace_file);
       Free(io->out_ancestral_file);
       Free(io->nt_or_cd);
       Free(io->run_id_string);
       Free(io->clade_list_file);
-      For(i,T_MAX_ALPHABET) Free(io->alphabet[i]);
+      for(i=0;i<T_MAX_ALPHABET;i++) Free(io->alphabet[i]);
       Free(io->alphabet);
       if(io->short_tax_names)
         {
-          For(i,io->size_tax_names)
+          for(i=0;i<io->size_tax_names;i++)
             {
               Free(io->short_tax_names[i]);
               Free(io->long_tax_names[i]);
@@ -774,9 +979,9 @@ void Free_St(supert_tree *st)
   int i;
 
   For(i,2*st->tree->n_otu-1)
-    Free(st->tree->a_edges[i]->nni);
+    Free_NNI(st->tree->a_edges[i]->nni);
 
-  For(i,st->n_part) Free(st->match_st_node_in_gt[i]);
+  for(i=0;i<st->n_part;i++) Free(st->match_st_node_in_gt[i]);
 
   Free(st->match_st_node_in_gt);
 
@@ -812,13 +1017,23 @@ void Free_Eigen(eigen *eigen_struct)
 void Free_One_Spr(t_spr *this_spr)
 {
   Free(this_spr->path);
-  Free(this_spr);
+  if(this_spr->l0) Free_Scalar_Dbl(this_spr->l0);
+  if(this_spr->l1) Free_Scalar_Dbl(this_spr->l1);
+  if(this_spr->l2) Free_Scalar_Dbl(this_spr->l2);
+  if(this_spr->v0) Free_Scalar_Dbl(this_spr->v0);
+  if(this_spr->v1) Free_Scalar_Dbl(this_spr->v1);
+  if(this_spr->v2) Free_Scalar_Dbl(this_spr->v2);
+  
+  if(this_spr->init_target_l) Free_Scalar_Dbl(this_spr->init_target_l);
+  if(this_spr->init_target_v) Free_Scalar_Dbl(this_spr->init_target_v);
+  
+  Free(this_spr);  
 }
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-void Free_Spr_List(t_tree *mixt_tree)
+void Free_Spr_List_One_Edge(t_tree *mixt_tree)
 {
   int i;
   t_tree *tree;
@@ -826,9 +1041,8 @@ void Free_Spr_List(t_tree *mixt_tree)
   tree = mixt_tree;
   do
     {
-      For(i,tree->size_spr_list+1) Free_One_Spr(tree->spr_list[i]);
-      Free(tree->spr_list);
-      Free_One_Spr(tree->best_spr);
+      for(i=0;i<tree->size_spr_list_one_edge+1;++i) Free_One_Spr(tree->spr_list_one_edge[i]);
+      Free(tree->spr_list_one_edge);
       tree = tree->next;
     }
   while(tree);
@@ -838,6 +1052,26 @@ void Free_Spr_List(t_tree *mixt_tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+void Free_Spr_List_All_Edge(t_tree *mixt_tree)
+{
+  int i;
+  t_tree *tree;
+
+  tree = mixt_tree;
+  do
+    {
+      for(i=0;i<tree->size_spr_list_all_edge+1;++i) Free_One_Spr(tree->spr_list_all_edge[i]);
+      Free(tree->spr_list_all_edge);
+      tree = tree->next;
+    }
+  while(tree);
+
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+
 void Free_Triplet(triplet *t)
 {
   int i,j,k;
@@ -849,35 +1083,35 @@ void Free_Triplet(triplet *t)
   Free(t->pi_cd);
   Free(t->pi_bd);
 
-  For(k,t->mod->ras->n_catg)
+  for(k=0;k<t->mod->ras->n_catg;++k)
     {
-      For(i,t->size)
-    {
-      For(j,t->size) Free(t->core[k][i][j]);
-      Free(t->core[k][i]);
-    }
+      for(i=0;i<t->size;++i)
+        {
+          for(j=0;j<t->size;++j) Free(t->core[k][i][j]);
+          Free(t->core[k][i]);
+        }
       Free(t->core[k]);
     }
   Free(t->core);
-
-  For(i,t->size)
+  
+  for(i=0;i<t->size;++i)
     {
-      For(j,t->size) Free(t->p_one_site[i][j]);
+      for(j=0;j<t->size;++j) Free(t->p_one_site[i][j]);
       Free(t->p_one_site[i]);
     }
   Free(t->p_one_site);
-
-  For(i,t->size)
+  
+  for(i=0;i<t->size;++i)
     {
-      For(j,t->size) Free(t->sum_p_one_site[i][j]);
+      for(j=0;j<t->size;++j) Free(t->sum_p_one_site[i][j]);
       Free(t->sum_p_one_site[i]);
     }
   Free(t->sum_p_one_site);
 
   Free_Eigen(t->eigen_struct);
-
+  
   if(t->next) Free_Triplet(t->next);
-
+  
   Free(t);
 }
 
@@ -888,7 +1122,7 @@ void Free_Triplet(triplet *t)
 void Free_Actual_CSeq(calign *data)
 {
   int i;
-  For(i,data->n_otu)
+  for(i=0;i<data->n_otu;i++)
     {
       Free(data->c_seq[i]->state);
       Free(data->c_seq[i]->d_state);
@@ -904,7 +1138,7 @@ void Free_Prefix_Tree(pnode *n, int size)
 {
   int i;
 
-  For(i,size)
+  for(i=0;i<size;i++)
     {
       if(n->next[i])
     {
@@ -939,7 +1173,7 @@ void Free_Nexus(option *io)
 {
   int i,j;
 
-  For(i,N_MAX_NEX_COM)
+  for(i=0;i<N_MAX_NEX_COM;i++)
     {
       For(j,io->nex_com_list[i]->nparm) Free_Nexus_Parm(io->nex_com_list[i]->parm[j]);
       Free(io->nex_com_list[i]->parm);
@@ -957,7 +1191,7 @@ void Free_Nexus_Com(nexcom **com)
 {
   int i;
 
-  For(i,N_MAX_NEX_COM)
+  for(i=0;i<N_MAX_NEX_COM;i++)
     {
       Free(com[i]->parm);
       Free(com[i]->name);
@@ -981,6 +1215,7 @@ void Free_Nexus_Parm(nexparm *parm)
 
 void XML_Free_XML_Tree(xml_node *node)
 {
+  if(!node) return;
   if(node->child) XML_Free_XML_Tree(node->child);
   if(node->next)  XML_Free_XML_Tree(node->next);
   XML_Free_XML_Node(node);
@@ -1042,7 +1277,7 @@ void MCMC_Free_MCMC(t_mcmc *mcmc)
   Free(mcmc->prev_run_move);
   Free(mcmc->acc_rate);
   Free(mcmc->tune_move);
-  For(i,mcmc->n_moves) Free(mcmc->move_name[i]);
+  for(i=0;i<mcmc->n_moves;i++) Free(mcmc->move_name[i]);
   Free(mcmc->move_name);
   Free(mcmc->ess_run);
   Free(mcmc->start_ess);
@@ -1065,7 +1300,7 @@ void M4_Free_M4_Model(m4 *m4mod)
 
   if(m4mod->o_mats)
     {
-      For(i,m4mod->n_h) Free(m4mod->o_mats[i]);
+      for(i=0;i<m4mod->n_h;i++) Free(m4mod->o_mats[i]);
       Free(m4mod->o_mats);
       Free(m4mod->h_mat);
       Free(m4mod->o_rr);
@@ -1087,9 +1322,11 @@ void RATES_Free_Rates(t_rate *rates)
 {
   if(rates->is_allocated == YES)
     {
+      int i;
       Free(rates->nd_r);
       Free(rates->br_r);
-      Free(rates->buff_r);
+      Free(rates->buff_nd_r);
+      Free(rates->buff_br_r);
       Free(rates->true_r);
       Free(rates->buff_t);
       Free(rates->nd_t);
@@ -1139,14 +1376,14 @@ void RATES_Free_Rates(t_rate *rates)
       Free(rates->survival_rank);
       Free(rates->survival_dur);
       Free(rates->calib_prob);
-      Free(rates->node_height_dens_log_norm_const_update);
-      Free(rates->curr_nd_for_cal);
       Free(rates->t_prior_min_ori);
       Free(rates->t_prior_max_ori);
       Free(rates->times_partial_proba);
       Free(rates->numb_calib_chosen);
+      for(i=0;i<rates->n_cal;i++) Free_Calib(rates->a_cal[i]);
+      Free(rates->a_cal);
+      Free(rates->model_name);
     }
-  Free_Calib(rates->calib);
   Free(rates);
 }
 
@@ -1156,8 +1393,15 @@ void RATES_Free_Rates(t_rate *rates)
 void Free_Calib(t_cal *cal)
 {
   if(!cal) return;
-  else Free_Calib(cal->next);
-  Free(cal);
+  else 
+    {    
+      int i;
+      for(i=0;i<cal->clade_list_size;i++) Free(cal->clade_list[i]);
+      Free(cal->clade_list);      
+      Free(cal->id);
+      Free(cal->alpha_proba_list);
+      Free(cal);
+    }
 }
 
 /*////////////////////////////////////////////////////////////
@@ -1173,7 +1417,7 @@ void Free_Geo(t_geo *t)
   Free(t->sorted_nd);
   Free(t->cov);
   Free(t->idx_loc_beneath);
-  For(i,t->ldscape_sz) Free_Geo_Coord(t->coord_loc[i]);
+  for(i=0;i<t->ldscape_sz;i++) Free_Geo_Coord(t->coord_loc[i]);
   Free(t->coord_loc);
   Free(t);
 }
@@ -1213,8 +1457,6 @@ void Free_Ldisk(t_ldsk *t)
     {
       Free(t->next);
       Free_Geo_Coord(t->coord);
-      /* if(t->min_coord) Free_Geo_Coord(t->min_coord); */
-      /* if(t->max_coord) Free_Geo_Coord(t->max_coord); */
       if(t->cpy_coord) Free_Geo_Coord(t->cpy_coord);
       Free(t);
     }
@@ -1227,7 +1469,7 @@ void Free_Ldisk(t_ldsk *t)
 void Free_Poly(t_poly *p)
 {
   int i;
-  For(i,p->n_poly_vert) Free_Geo_Coord(p->poly_vert[i]);
+  for(i=0;i<p->n_poly_vert;i++) Free_Geo_Coord(p->poly_vert[i]);
   Free(p->poly_vert);
   Free(p);
 }
@@ -1237,9 +1479,88 @@ void Free_Poly(t_poly *p)
 
 void Free_Mmod(t_phyrex_mod *mmod)
 {
+  if(mmod == NULL) return;
   Free_Geo_Coord(mmod->lim);
   Free(mmod);
 }
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void JSON_Free_Array(json_a *a)
+{
+  if(a->object) JSON_Free_Object(a->object);
+  Free(a);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void JSON_Free_Object(json_o *o)
+{
+  if(o->kv) JSON_Free_KeyVal(o->kv);
+  if(o->next) JSON_Free_Object(o->next);
+  Free(o);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void JSON_Free_KeyVal(json_kv *kv)
+{
+  if(kv->key) Free(kv->key);
+  if(kv->value) Free(kv->value);
+  if(kv->object) JSON_Free_Object(kv->object);
+  if(kv->array) JSON_Free_Array(kv->array);
+  if(kv->next) JSON_Free_KeyVal(kv->next);
+  Free(kv);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Free_NNI(t_nni *t)
+{
+  if(t->init_l) Free_Scalar_Dbl(t->init_l);
+  if(t->init_v) Free_Scalar_Dbl(t->init_v);
+
+  if(t->best_l) Free_Scalar_Dbl(t->best_l);
+  if(t->best_v) Free_Scalar_Dbl(t->best_v);
+  
+  if(t->l0) Free_Scalar_Dbl(t->l0);
+  if(t->v0) Free_Scalar_Dbl(t->v0);
+
+  if(t->l1) Free_Scalar_Dbl(t->l1);
+  if(t->v1) Free_Scalar_Dbl(t->v1);
+
+  if(t->l2) Free_Scalar_Dbl(t->l2);
+  if(t->v2) Free_Scalar_Dbl(t->v2);
+  
+  Free(t);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+/* Matrices used in transfer bootstrap computation (tbe.c) */
+void Free_TBE_Matrices(int n_otu,  short unsigned*** i_matrix, short unsigned*** c_matrix,
+		       short unsigned*** hamming, short unsigned** min_dist,
+		       short unsigned**  min_dist_edge, int** cluster_sizes){
+  int i;
+  int nb_edges = 2*n_otu-3;
+  for (i=0; i<nb_edges; i++) {
+    free((*c_matrix)[i]);
+    free((*i_matrix)[i]);
+    free((*hamming)[i]);
+  }
+  free((*c_matrix));
+  free((*i_matrix));
+  free((*hamming));
+  free((*min_dist));
+  free((*min_dist_edge));
+  free((*cluster_sizes));
+}
+
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
@@ -1247,3 +1568,7 @@ void Free_Mmod(t_phyrex_mod *mmod)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
