@@ -31,7 +31,7 @@ void Usage()
   char *cha;
 
 
-  cha =getenv("OS");
+  cha = getenv("OS");
 
   if(cha!=NULL) 
     {
@@ -126,8 +126,12 @@ void Usage()
   PhyML_Printf("\t\t%sint%s = -4: SH-like branch supports alone.\n",LINE,FLAT);
   PhyML_Printf("\t\t%sint%s = -5: (default) approximate Bayes branch supports.\n",LINE,FLAT);
   PhyML_Printf("\n");
-  #endif
 
+  PhyML_Printf("%s\n\t--tbe%s\n",BOLD,FLAT);
+  PhyML_Printf("\t\tComputes TBE instead of FBP (standard) bootstrap support\n");
+  PhyML_Printf("\t\tHas no effect with -b <= 0\n");
+
+  #endif
 
   PhyML_Printf("%s\n\t-m (or --model) %smodel%s\n",BOLD,LINE,FLAT);
   PhyML_Printf("\t\tmodel%s : substitution model name.\n",FLAT);
@@ -178,7 +182,7 @@ void Usage()
   PhyML_Printf("%s\t\t- %sAmino-acid%s sequences: (Model) the equilibrium amino-acid frequencies are estimated using\n"
 "\t\t the frequencies defined by the substitution model.\n",FLAT,LINE,FLAT);
   PhyML_Printf("\n");
-  PhyML_Printf("\t\t%s\"fA,fC,fG,fT\"%s : only valid for nucleotide-based models. fA, fC, fG and fT are floating numbers that \n",LINE,FLAT);
+  PhyML_Printf("\t\t%sfA,fC,fG,fT%s : only valid for nucleotide-based models. fA, fC, fG and fT are floating numbers that \n",LINE,FLAT);
   PhyML_Printf("\t\t correspond to the frequencies of A, C, G and T respectively (WARNING: do not use any blank space between\n");
   PhyML_Printf("\t\t your values of nucleotide frequencies, only commas!)\n");
   PhyML_Printf("\n");
