@@ -1,7 +1,7 @@
 /*
 
-PhyML:  a program that  computes maximum likelihood phyLOGenies from
-DNA or AA homoLOGous sequences.
+PhyML:  a program that  computes maximum likelihood phylogenies from
+DNA or AA homologous sequences.
 
 Copyright (C) Stephane Guindon. Oct 2003 onward.
 
@@ -52,6 +52,7 @@ int main(int argc, char **argv)
   char *most_likely_tree=NULL;
   int orig_random_input_tree;
 
+  
 #ifdef MPI
   int rc;
   rc = MPI_Init(&argc,&argv);
@@ -81,7 +82,6 @@ int main(int argc, char **argv)
       return(0);
     }
 
-
 #ifdef EVOLVE
   io->colalias = NO;
 #endif
@@ -105,7 +105,7 @@ int main(int argc, char **argv)
       io->n_trees     = MIN(io->n_trees,io->n_data_sets);
     }
 
-  For(num_data_set,io->n_data_sets)
+  for(num_data_set=0;num_data_set<io->n_data_sets;num_data_set++)
     {
       best_lnL = UNLIKELY;
       Get_Seq(io);
@@ -115,16 +115,17 @@ int main(int argc, char **argv)
       mod = io->mod;
       orig_random_input_tree = io->mod->s_opt->random_input_tree;
       
+
       if(io->data)
         {
           if(io->n_data_sets > 1) PhyML_Printf("\n. Data set [#%d]\n",num_data_set+1);
           cdata = Compact_Data(io->data,io);
-
+          
           Free_Seq(io->data,cdata->n_otu);
 
           for(num_tree=(io->n_trees == 1)?(0):(num_data_set);num_tree < io->n_trees;num_tree++)
             {
-              if(!io->mod->s_opt->random_input_tree) io->mod->s_opt->n_rand_starts = 1;
+              if(io->mod->s_opt->random_input_tree == NO) io->mod->s_opt->n_rand_starts = 1;
 
               if(orig_random_input_tree == YES && io->n_trees > 1)
                 {
@@ -132,7 +133,7 @@ int main(int argc, char **argv)
                   Exit("\n");
                 }
 
-              For(num_rand_tree,io->mod->s_opt->n_rand_starts)
+              for(num_rand_tree=0;num_rand_tree<io->mod->s_opt->n_rand_starts;num_rand_tree++)
                 {
                   if((io->mod->s_opt->random_input_tree) && (io->mod->s_opt->topo_search != NNI_MOVE))
                     if(!io->quiet) PhyML_Printf("\n\n. [Random start %3d/%3d]",num_rand_tree+1,io->mod->s_opt->n_rand_starts);
@@ -141,6 +142,8 @@ int main(int argc, char **argv)
 
                   if(io->mod->use_m4mod) M4_Init_Model(mod->m4mod,cdata,mod);
 
+                  Remove_Duplicates(cdata,mod,io);
+
                   //Make the initial tree
                   switch(io->in_tree)
                     {
@@ -173,11 +176,12 @@ int main(int argc, char **argv)
                       Get_Bip(io->cstr_tree->a_nodes[0],
                               io->cstr_tree->a_nodes[0]->v[0],
                               io->cstr_tree);
-                      if(!tree->has_branch_lengths) Add_BioNJ_Branch_Lengths(tree,cdata,mod);                      
+                      if(tree->has_branch_lengths == NO) Add_BioNJ_Branch_Lengths(tree,cdata,mod,NULL);
                     }
 
                   if(!tree) continue;
 
+                  
                   time(&t_beg);
                   time(&(tree->t_beg));
 
@@ -205,11 +209,11 @@ int main(int argc, char **argv)
                       Exit("\n");
                     }
 
-
                   Prepare_Tree_For_Lk(tree);
                   Br_Len_Not_Involving_Invar(tree);
                   Unscale_Br_Len_Multiplier_Tree(tree);
 
+
 #ifdef BEAGLE
                   if(mod->bootstrap == YES)
                     {
@@ -224,20 +228,21 @@ int main(int argc, char **argv)
 #endif
 
 #ifdef PHYML
-                  if(io->in_tree == 1) Spr_Pars(tree);
+                  if(tree->io->print_json_trace == YES) JSON_Tree_Io(tree,tree->io->fp_out_json_trace); 
+                                                     
 
-                  /* /\* ******************** *\/ */
-                  /* Sample_Ancestral_Seq(NO,YES,tree); */
-                  /* tree->mod->augmented = YES; */
-                  /* Lk(NULL,tree); */
-                  /* /\* ******************** *\/ */
-                                   
-		  if(tree->mod->s_opt->opt_topo)
+                  
+                  Switch_Eigen(YES,tree->mod);                 
+                  Lk(NULL,tree);
+                  Switch_Eigen(NO,tree->mod);
+
+
+                  tree->n_tot_bl_opt = 0;
+
+                  if(tree->mod->s_opt->opt_topo)
 		    {
-		      if(tree->mod->s_opt->topo_search      == NNI_MOVE) Simu_Loop(tree);
-		      else if(tree->mod->s_opt->topo_search == SPR_MOVE) Speed_Spr_Loop(tree);
-		      else                                               Best_Of_NNI_And_SPR(tree);
-
+                      Global_Spr_Search(tree);
+                      
                       if(tree->n_root) Add_Root(tree->a_edges[0],tree);
                     }
                   else
@@ -246,16 +251,13 @@ int main(int argc, char **argv)
                       tree->b_inst = create_beagle_instance(tree, io->quiet, io);
 #endif
                       //Optimize Branch lengths?
-                      if(tree->mod->s_opt->opt_subst_param || tree->mod->s_opt->opt_bl) {
-                        Round_Optimize(tree,tree->data,ROUND_MAX);
-                      } 
-                      else { //No topology or branch length optimizations
-                        Lk(NULL,tree);
-                      }
+                      /* if(tree->mod->s_opt->opt_subst_param || tree->mod->s_opt->opt_bl)  */
+                        Round_Optimize(tree,ROUND_MAX);
+                       
                     }
 
-
-                  if(tree->mod->gamma_mgf_bl) Optimum_Root_Position_IL_Model(tree);
+                  
+                  if(tree->mod->gamma_mgf_bl) Best_Root_Position_IL_Model(tree);
 
                   Set_Both_Sides(YES,tree);
                   Lk(NULL,tree);
@@ -264,7 +266,7 @@ int main(int argc, char **argv)
                   PhyML_Printf("\n\n. Log likelihood of the current tree: %f.",tree->c_lnL);
 
                           
-                  if(tree->io->ancestral == YES) ML_Ancestral_Sequences(tree);
+                  if(tree->io->ancestral == YES) Ancestral_Sequences(tree,YES);
                   
                   /* Build_Distrib_Number_Of_Diff_States_Under_Model(tree); */
                   /* Exit("\n"); */
@@ -316,13 +318,19 @@ int main(int argc, char **argv)
 #ifdef BEAGLE
                   finalize_beagle_instance(tree);
 #endif
-                  Free_Spr_List(tree);
+                  Free_One_Spr(tree->best_spr);
+                  Free_Spr_List_One_Edge(tree);
+                  Free_Spr_List_All_Edge(tree);
                   Free_Triplet(tree->triplet_struct);
                   Free_Tree_Pars(tree);
                   Free_Tree_Lk(tree);
                   Free_Tree(tree);
                 } //Tree done
 
+              if(io->n_data_sets == 1) rewind(io->fp_out_tree);
+              if(most_likely_tree) PhyML_Fprintf(io->fp_out_tree,"%s\n",most_likely_tree);
+
+
               /* Launch bootstrap analysis */
               if(mod->bootstrap)
                 {
@@ -330,7 +338,7 @@ int main(int argc, char **argv)
 
 #ifdef MPI
                   MPI_Bcast (most_likely_tree, strlen(most_likely_tree)+1, MPI_CHAR, 0, MPI_COMM_WORLD);
-                  if(!io->quiet)  PhyML_Printf("\n\n. The bootstrap analysis will use %d CPUs.",Global_numTask);
+                  if(!io->quiet)  PhyML_Printf("\n\n. The bootstrap analysis will use %d CPU%c.",Global_numTask,Global_numTask>1?'s':'\0');
 #endif
                   
                   most_likely_tree = Bootstrap_From_String(most_likely_tree,cdata,mod,io);
@@ -351,7 +359,7 @@ int main(int argc, char **argv)
 
               if(io->n_trees > 1 && io->n_data_sets > 1) break;
             }
-            Free_Cseq(cdata);
+            Free_Calign(cdata);
         }
     else
         {
@@ -378,6 +386,7 @@ int main(int argc, char **argv)
   if(io->fp_out_trees)          fclose(io->fp_out_trees);
   if(io->fp_out_stats)          fclose(io->fp_out_stats);
   if(io->fp_out_trace)          fclose(io->fp_out_trace);
+  if(io->fp_out_json_trace)     fclose(io->fp_out_json_trace);
 
   if(io->fp_in_constraint_tree != NULL) Free_Tree(io->cstr_tree);
   Free_Input(io);
@@ -408,13 +417,13 @@ int main(int argc, char **argv)
   return 1;
 }
 
-#elif(PHYTIME)
-#include "times.h"
-int main(int argc, char **argv)
-{
-  TIMES_main(argc, argv);
-  return 1;
-}
+/* #elif(PHYTIME) */
+/* #include "times.h" */
+/* int main(int argc, char **argv) */
+/* { */
+/*   TIMES_main(argc, argv); */
+/*   return 1; */
+/* } */
 
 #elif(PHYCONT)
 #include "continuous.h"
@@ -437,12 +446,22 @@ int main(int argc, char **argv)
   tree1 = Read_Tree_File_Phylip(fp_tree1);
   tree2 = Read_Tree_File_Phylip(fp_tree2);
 
-  Match_Nodes_In_Small_Tree(tree1,tree2);
 
-  For(i,2*tree1->n_otu-2)
-    {
-      printf("\n. Node %d in tree1 matches node %d in tree2",i,(tree1->noeud[i]->match_node)?(tree1->noeud[i]->match_node->num):(-1));
-    }
+  Prune_Tree(tree1,tree2);
+  Prune_Tree(tree2,tree1);
+  
+
+  PhyML_Printf("%s\n%s",
+               Write_Tree(tree1,NO),
+               Write_Tree(tree2,NO));
+               
+  
+  /* Match_Nodes_In_Small_Tree(tree1,tree2); */
+
+  /* For(i,2*tree1->n_otu-2) */
+  /*   { */
+  /*     printf("\n. Node %d in tree1 matches node %d in tree2",i,(tree1->noeud[i]->match_node)?(tree1->noeud[i]->match_node->num):(-1)); */
+  /*   } */
 
 
 
@@ -569,6 +588,14 @@ int main(int argc, char **argv)
   return 1;
 }
 
+#elif(PHYTIME)
+#include "date.h"
+int main(int argc, char **argv)
+{
+  DATE_Main(argc,argv);
+  return 1;
+}
+
 #elif(CHECKPOINT)
 #include "checkpoint.h"
 int main(int argc, char **argv)
