@@ -25,22 +25,49 @@ void Make_Tree_4_Lk(t_tree *tree, calign *cdata, int n_site)
   tree->unscaled_site_lk_cat = (phydbl *)mCalloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->n_pattern,sizeof(phydbl));
   tree->fact_sum_scale       = (int *)mCalloc(tree->n_pattern,sizeof(int));
 
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void **)&tree->eigen_lr_left,BYTE_ALIGN,(size_t)MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->eigen_lr_rght,BYTE_ALIGN,(size_t)MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->dot_prod,BYTE_ALIGN,(size_t)tree->n_pattern*tree->mod->ns*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->expl,BYTE_ALIGN,(size_t)3*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  tree->eigen_lr_left = _aligned_malloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+  tree->eigen_lr_rght = _aligned_malloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+  tree->dot_prod = _aligned_malloc(tree->n_pattern*tree->mod->ns*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*sizeof(phydbl),BYTE_ALIGN);
+  tree->expl = _aligned_malloc(3*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+  tree->eigen_lr_left = (phydbl *)mCalloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+  tree->eigen_lr_rght = (phydbl *)mCalloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+  tree->dot_prod = (phydbl *)mCalloc(tree->n_pattern*tree->mod->ns*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes),sizeof(phydbl));
+  tree->expl = (phydbl *)mCalloc(3*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+#endif
+
+
   tree->log_lks_aLRT = (phydbl **)mCalloc(3,sizeof(phydbl *));
-  For(i,3) tree->log_lks_aLRT[i] = (phydbl *)mCalloc(tree->data->init_len,sizeof(phydbl));
+  for(i=0;i<3;i++) tree->log_lks_aLRT[i] = (phydbl *)mCalloc(tree->data->init_len,sizeof(phydbl));
 
-  For(i,2*tree->n_otu-1) Make_Edge_NNI(tree->a_edges[i]);
+  for(i=0;i<2*tree->n_otu-1;++i) Make_Edge_NNI(tree->a_edges[i]);
 
+  Make_Extra_Edge_Lk(tree);
+  
   if(tree->is_mixt_tree == NO)
     {
-      For(i,2*tree->n_otu-1) Make_Edge_Lk(tree->a_edges[i],tree);
-      For(i,2*tree->n_otu-2) Make_Node_Lk(tree->a_nodes[i]);
+      for(i=0;i<2*tree->n_otu-1;++i) Make_Edge_Lk(tree->a_edges[i],tree);
+      for(i=0;i<2*tree->n_otu-2;++i) Make_Node_Lk(tree->a_nodes[i]);
+      for(i=0;i<2*tree->n_otu-1;++i) Make_Edge_Loc(tree->a_edges[i],tree);
 
-      if(tree->mod->s_opt->greedy)
-        Init_P_Lk_Tips_Double(tree);
-      else
-        Init_P_Lk_Tips_Int(tree);
+      Init_Partial_Lk_Tips_Double(tree);
+      Init_Partial_Lk_Loc(tree);
 
-      Init_P_Lk_Loc(tree);
+      if(tree->n_root != NULL)
+        {
+          Free_Edge_Lk_Rght(tree->n_root->b[1]);
+          Free_Edge_Lk_Rght(tree->n_root->b[2]);
+          Free_Edge_Loc_Rght(tree->n_root->b[1]);
+          Free_Edge_Loc_Rght(tree->n_root->b[2]);
+        }
     }
 }
 
@@ -53,9 +80,17 @@ void Make_Tree_4_Pars(t_tree *tree, calign *cdata, int n_site)
   int i;
   tree->site_pars = (int *)mCalloc(tree->n_pattern,sizeof(int));
   tree->step_mat = (int *)mCalloc(tree->mod->ns * tree->mod->ns,sizeof(int));
+
   For(i,2*tree->n_otu-1) Make_Edge_Pars(tree->a_edges[i],tree);
   Init_Ui_Tips(tree);
-  Init_P_Pars_Tips(tree); /* Must be called after Init_Ui_Tips is called */
+  Init_Partial_Pars_Tips(tree); /* Must be called after Init_Ui_Tips is called */
+
+  if(tree->n_root)
+    {
+      Free_Edge_Pars_Rght(tree->a_edges[2*tree->n_otu-3]);
+      Free_Edge_Pars_Rght(tree->a_edges[2*tree->n_otu-2]);
+    }
+
   Get_Step_Mat(tree);
 }
 
@@ -66,14 +101,14 @@ void Make_All_Edges_Lk(t_node *a, t_node *d, t_tree *tree)
 {
   int i;
 
-  For(i,3) 
+  for(i=0;i<3;i++) 
     if((a->v[i]) && (a->v[i] == d)) 
       Make_Edge_Lk(a->b[i],tree);
 
   if(d->tax) return;
   else
     {
-      For(i,3)
+      for(i=0;i<3;i++)
         {
           if(d->v[i] != a && d->b[i] != tree->e_root)
             Make_All_Edges_Lk(d,d->v[i],tree);
@@ -136,7 +171,7 @@ t_edge *Make_Edge_Light(t_node *a, t_node *d, int num)
 
       b->l->v             = a->l[b->l_r];
       if(a->tax) b->l->v  = a->l[b->r_l];
-      b->l_old->v            = b->l->v;
+      b->l_old->v         = b->l->v;
     }
   else
     {
@@ -163,7 +198,7 @@ void Make_Edge_Pars(t_edge *b, t_tree *tree)
 void Make_Edge_Pars_Left(t_edge *b, t_tree *tree)
 {
   b->pars_l = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
-  b->ui_l = (unsigned int *)mCalloc(tree->data->crunch_len,sizeof(unsigned int));
+  b->ui_l = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
   b->p_pars_l = (int *)mCalloc(tree->data->crunch_len*tree->mod->ns,sizeof(int ));
   b->n_diff_states_l = (int *)mCalloc(tree->mod->ns,sizeof(int ));
 }
@@ -174,7 +209,7 @@ void Make_Edge_Pars_Left(t_edge *b, t_tree *tree)
 void Make_Edge_Pars_Rght(t_edge *b, t_tree *tree)
 {
   b->pars_r = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
-  b->ui_r = (unsigned int *)mCalloc(tree->data->crunch_len,sizeof(unsigned int));
+  b->ui_r = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
   b->p_pars_r = (int *)mCalloc(tree->data->crunch_len*tree->mod->ns,sizeof(int ));
   b->n_diff_states_r = (int *)mCalloc(tree->mod->ns,sizeof(int ));
 }
@@ -192,7 +227,18 @@ void Make_Edge_Lk(t_edge *b, t_tree *tree)
 
   b->l_old->v = b->l->v;
 
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void *)&b->Pij_rr,BYTE_ALIGN,(size_t)tree->mod->ras->n_catg*tree->mod->ns*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void *)&b->tPij_rr,BYTE_ALIGN,(size_t)tree->mod->ras->n_catg*tree->mod->ns*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  b->Pij_rr = _aligned_malloc(tree->mod->ras->n_catg*tree->mod->ns*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+  b->tPij_rr = _aligned_malloc(tree->mod->ras->n_catg*tree->mod->ns*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
   b->Pij_rr = (phydbl *)mCalloc(tree->mod->ras->n_catg*tree->mod->ns*tree->mod->ns,sizeof(phydbl));
+  b->tPij_rr = (phydbl *)mCalloc(tree->mod->ras->n_catg*tree->mod->ns*tree->mod->ns,sizeof(phydbl));
+#endif
 
   Make_Edge_Lk_Left(b,tree);
   Make_Edge_Lk_Rght(b,tree);
@@ -218,13 +264,30 @@ void Make_Edge_Lk_Left(t_edge *b, t_tree *tree)
     {
       if((!b->left->tax) || (tree->mod->s_opt->greedy))
         {
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+          if(posix_memalign((void **)&b->p_lk_left,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+          b->p_lk_left = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
           b->p_lk_left = (phydbl *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+#endif
           b->p_lk_tip_l = NULL;
         }
       else if(b->left->tax)
         {
           b->p_lk_left   = NULL;
-          b->p_lk_tip_l  = (short int *)mCalloc(tree->data->crunch_len*tree->mod->ns,sizeof(short int ));
+          
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+          if(posix_memalign((void **)&b->p_lk_tip_l,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+          b->p_lk_tip_l = _aligned_malloc(tree->data->crunch_len*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+          b->p_lk_tip_l  = (phydbl *)mCalloc(tree->data->crunch_len*tree->mod->ns,sizeof(phydbl));
+#endif
         }
     }
   else
@@ -236,12 +299,19 @@ void Make_Edge_Lk_Left(t_edge *b, t_tree *tree)
   if(b->num >= 2*tree->n_otu-3)
     {
       b->sum_scale_left = (int *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes),sizeof(int));
+
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+      if(posix_memalign((void **)&b->p_lk_left,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+      b->p_lk_left = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
       b->p_lk_left      = (phydbl *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+#endif
     }
 
-
   b->patt_id_left  = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
-  b->p_lk_loc_left = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
 }
 
 //////////////////////////////////////////////////////////////
@@ -265,13 +335,29 @@ void Make_Edge_Lk_Rght(t_edge *b, t_tree *tree)
     {
       if((!b->rght->tax) || (tree->mod->s_opt->greedy))
         {
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+	  if(posix_memalign((void **)&b->p_lk_rght,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else          
+          b->p_lk_rght = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
           b->p_lk_rght = (phydbl *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+#endif
           b->p_lk_tip_r = NULL;
         }
       else if(b->rght->tax)
         {
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+          if(posix_memalign((void **)&b->p_lk_tip_r,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+          b->p_lk_tip_r = _aligned_malloc(tree->data->crunch_len*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+          b->p_lk_tip_r  = (phydbl *)mCalloc(tree->data->crunch_len*tree->mod->ns,sizeof(phydbl));
+#endif
           b->p_lk_rght = NULL;
-          b->p_lk_tip_r  = (short int *)mCalloc(tree->data->crunch_len*tree->mod->ns,sizeof(short int));
         }
     }
   else
@@ -283,16 +369,117 @@ void Make_Edge_Lk_Rght(t_edge *b, t_tree *tree)
   if(b->num >= 2*tree->n_otu-3)
     {
       b->sum_scale_rght = (int *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes),sizeof(int));
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+      if(posix_memalign((void **)&b->p_lk_rght,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+      b->p_lk_rght      = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+
+
+#else
       b->p_lk_rght      = (phydbl *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+#endif
     }
 
   b->patt_id_rght  = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Make_Extra_Edge_Lk(t_tree *tree)
+{
+  int ns = tree->mod->ns;
+
+  tree->div_post_pred_extra_0 = (short int *)mCalloc(ns,sizeof(short int));
+
+  tree->sum_scale_cat_extra_0 = (int *)mCalloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes),sizeof(int));
+
+  tree->sum_scale_extra_0 = (int *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes),sizeof(int));
+
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void **)&tree->p_lk_extra_0,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  tree->p_lk_extra_0 = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+  tree->p_lk_extra_0 = (phydbl *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+#endif
+
+  
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void **)&tree->p_lk_tip_extra_0,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  tree->p_lk_tip_extra_0 = _aligned_malloc(tree->data->crunch_len*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+  tree->p_lk_tip_extra_0  = (phydbl *)mCalloc(tree->data->crunch_len*tree->mod->ns,sizeof(phydbl));
+#endif
+        
+  tree->patt_id_extra_0  = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
+
+  tree->div_post_pred_extra_1 = (short int *)mCalloc(ns,sizeof(short int));
+
+  tree->sum_scale_cat_extra_1 = (int *)mCalloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes),sizeof(int));
+
+  tree->sum_scale_extra_1 = (int *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes),sizeof(int));
+
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void **)&tree->p_lk_extra_1,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  tree->p_lk_extra_1 = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+  tree->p_lk_extra_1 = (phydbl *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns,sizeof(phydbl));
+#endif
+
+  
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void **)&tree->p_lk_tip_extra_1,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  tree->p_lk_tip_extra_1 = _aligned_malloc(tree->data->crunch_len*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+  tree->p_lk_tip_extra_1  = (phydbl *)mCalloc(tree->data->crunch_len*tree->mod->ns,sizeof(phydbl));
+#endif
+        
+  tree->patt_id_extra_1  = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
+
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Make_Edge_Loc(t_edge *b, t_tree *tree)
+{
+  Make_Edge_Loc_Left(b,tree);
+  Make_Edge_Loc_Rght(b,tree);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Make_Edge_Loc_Rght(t_edge *b, t_tree *tree)
+{
   b->p_lk_loc_rght = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
 }
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+void Make_Edge_Loc_Left(t_edge *b, t_tree *tree)
+{
+  b->p_lk_loc_left = (int *)mCalloc(tree->data->crunch_len,sizeof(int));
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 void Make_Edge_NNI(t_edge *b)
 {
   b->nni    = Make_NNI();
@@ -304,11 +491,12 @@ void Make_Edge_NNI(t_edge *b)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
-nni *Make_NNI()
+t_nni *Make_NNI()
 {
-  nni *a_nni;
-  a_nni = (nni *)mCalloc(1,sizeof(nni ));
+  t_nni *a_nni;
+
+  a_nni = (t_nni *)mCalloc(1,sizeof(t_nni));
+  
   Init_NNI(a_nni);
   return a_nni;
 }
@@ -327,7 +515,8 @@ t_node *Make_Node_Light(int num)
   n->score    = (phydbl *)mCalloc(3,sizeof(phydbl));
   n->s_ingrp  = (int *)mCalloc(3,sizeof(int));
   n->s_outgrp = (int *)mCalloc(3,sizeof(int));
-
+  n->cal      = (t_cal **)mCalloc(MAX_N_CAL,sizeof(t_cal *));
+ 
   Init_Node_Light(n,num);
 
   return n;
@@ -351,7 +540,7 @@ nexcom **Make_Nexus_Com()
 
   com = (nexcom **)mCalloc(N_MAX_NEX_COM,sizeof(nexcom *));
 
-  For(i,N_MAX_NEX_COM)
+  for(i=0;i<N_MAX_NEX_COM;i++)
     {
       com[i]       = (nexcom *)mCalloc(1,sizeof(nexcom));
       com[i]->name = (char *)mCalloc(T_MAX_NEX_COM,sizeof(char));
@@ -396,7 +585,7 @@ matrix *Make_Mat(int n_otu)
   mat->tip_node = (t_node **)mCalloc(n_otu,sizeof(t_node *));
 
 
-  For(i,n_otu)
+  for(i=0;i<n_otu;i++)
     {
       mat->P[i]    = (phydbl *)mCalloc(n_otu,sizeof(phydbl));
       mat->Q[i]    = (phydbl *)mCalloc(n_otu,sizeof(phydbl));
@@ -411,7 +600,7 @@ matrix *Make_Mat(int n_otu)
 //////////////////////////////////////////////////////////////
 
 t_tree *Make_Tree_From_Scratch(int n_otu, calign *data)
-{
+{  
   t_tree *tree;
 
   tree = Make_Tree(n_otu);
@@ -424,27 +613,27 @@ t_tree *Make_Tree_From_Scratch(int n_otu, calign *data)
       Copy_Tax_Names_To_Tip_Labels(tree,data);
       tree->data = data;
     }
-
+  
+  
 #ifdef BEAGLE
   //offset the branch's partial indices because BEAGLE insists on first storing the tips/taxa
   int num_branches = 2*tree->n_otu-1;
   int i;
   for(i=0;i<2*tree->n_otu-1;++i)
-  {
+    {
       //For edgeX, its "left" partial lies at index `num_tax + edgeX->num"
       tree->a_edges[i]->p_lk_left_idx = tree->n_otu + tree->a_edges[i]->p_lk_left_idx;
       //For edgeX, its "right" partial lies at index `num_tax + edgeX->num + num_branches"
       tree->a_edges[i]->p_lk_rght_idx = tree->n_otu + tree->a_edges[i]->p_lk_left_idx + num_branches;
-  }
+    }
 #endif
-
+  
   return tree;
 }
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 t_tree *Make_Tree(int n_otu)
 {
   t_tree *tree;
@@ -472,7 +661,7 @@ void Make_All_Tree_Nodes(t_tree *tree)
 
   tree->a_nodes = (t_node **)mCalloc(2*tree->n_otu-1,sizeof(t_node *));
 
-  For(i,2*tree->n_otu-1)
+  for(i=0;i<2*tree->n_otu-1;++i)
     {
       tree->a_nodes[i] = (t_node *)Make_Node_Light(i);
       if(i < tree->n_otu) tree->a_nodes[i]->tax = 1;
@@ -487,32 +676,26 @@ void Make_All_Tree_Edges(t_tree *tree)
 {
   int i;
   tree->a_edges = (t_edge **)mCalloc(2*tree->n_otu-1,sizeof(t_edge *));
-  For(i,2*tree->n_otu-1) tree->a_edges[i] = (t_edge *)Make_Edge_Light(NULL,NULL,i);
+  for(i=0;i<2*tree->n_otu-1;++i) tree->a_edges[i] = (t_edge *)Make_Edge_Light(NULL,NULL,i);
 }
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-calign *Make_Cseq(int n_otu, int crunch_len, int state_len, int init_len, char **sp_names)
+calign *Make_Calign(int n_otu, int crunch_len, int state_len, int init_len, char **sp_names)
 {
   calign *cdata;
   int j;
 
-  cdata           = (calign *)mCalloc(1,sizeof(calign));
-  cdata->n_otu    = n_otu;
-  cdata->c_seq    = (align **)mCalloc(n_otu,sizeof(align *));
-  cdata->b_frq    = (phydbl *)mCalloc(T_MAX_ALPHABET,sizeof(phydbl));
-  cdata->wght     = (int *)mCalloc(crunch_len,sizeof(int));
-  cdata->ambigu   = (short int *)mCalloc(crunch_len,sizeof(short int));
-  cdata->invar    = (short int *)mCalloc(crunch_len,sizeof(short int));
-  cdata->sitepatt = (int *)mCalloc(init_len,sizeof(int ));
-  cdata->format   = 0;
+  cdata                = (calign *)mCalloc(1,sizeof(calign));
+  cdata->c_seq         = (align **)mCalloc(n_otu,sizeof(align *));
+  cdata->obs_state_frq = (phydbl *)mCalloc(T_MAX_ALPHABET,sizeof(phydbl));
+  cdata->wght          = (phydbl *)mCalloc(crunch_len,sizeof(phydbl));
+  cdata->ambigu        = (short int *)mCalloc(crunch_len,sizeof(short int));
+  cdata->invar         = (short int *)mCalloc(crunch_len,sizeof(short int));
+  cdata->sitepatt      = (int *)mCalloc(init_len,sizeof(int ));
 
-  cdata->crunch_len = crunch_len;
-  cdata->init_len   = init_len;
-  cdata->obs_pinvar = .0;
-
-  For(j,n_otu)
+  for(j=0;j<n_otu;j++)
     {
       cdata->c_seq[j]            = (align *)mCalloc(1,sizeof(align));
       cdata->c_seq[j]->name      = (char *)mCalloc((int)(strlen(sp_names[j])+1),sizeof(char));
@@ -560,7 +743,7 @@ void Make_Custom_Model(t_mod *mod)
       PhyML_Printf("\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
       Warn_And_Exit("");
     }
-
+  
   if(!mod->r_mat->rr->v)
     mod->r_mat->rr->v = (phydbl *)mCalloc(mod->ns*(mod->ns-1)/2,sizeof(phydbl));
 
@@ -619,10 +802,6 @@ t_mod *Make_Model_Basic()
   mod->mr                     = (scalar_dbl *)mCalloc(1,sizeof(scalar_dbl));
   Init_Scalar_Dbl(mod->mr);
 
-  mod->user_b_freq            = (vect_dbl *)mCalloc(1,sizeof(vect_dbl));
-  Init_Vect_Dbl(0,mod->user_b_freq);
-  mod->user_b_freq->v         = (phydbl *)mCalloc(T_MAX_OPTION,sizeof(phydbl));
-
   mod->e_frq_weight           = (scalar_dbl *)mCalloc(1,sizeof(scalar_dbl));
   Init_Scalar_Dbl(mod->e_frq_weight);
 
@@ -631,6 +810,7 @@ t_mod *Make_Model_Basic()
 
   mod->aa_rate_mat_file       = Make_String(T_MAX_FILE);
   Init_String(mod->aa_rate_mat_file);
+
   return mod;
 }
 
@@ -650,25 +830,35 @@ void Make_Model_Complete(t_mod *mod)
 
   mod->Pij_rr = (vect_dbl *)mCalloc(1,sizeof(vect_dbl));
   Init_Vect_Dbl(0,mod->Pij_rr);
+
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void **)&mod->Pij_rr->v,BYTE_ALIGN,(size_t)mod->ras->n_catg*mod->ns*mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  mod->Pij_rr->v = _aligned_malloc(mod->ras->n_catg*mod->ns*mod->ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
   mod->Pij_rr->v = (phydbl *)mCalloc(mod->ras->n_catg*mod->ns*mod->ns,sizeof(phydbl));
+#endif
 
   mod->eigen     = (eigen *)Make_Eigen_Struct(mod->ns);
-
+  
   // If r_mat (e_frq) are not NULL, then they have been created elsewhere and affected.
   if(!mod->r_mat)
     {
       mod->r_mat = (t_rmat *)Make_Rmat(mod->ns);
       Init_Rmat(mod->r_mat);
     }
+
   if(!mod->e_frq)
     {
       mod->e_frq = (t_efrq *)Make_Efrq(mod->ns);
-      Init_Efrq(mod->e_frq);
+      Init_Efrq(NULL,mod->e_frq);
     }
 
   Make_RAS_Complete(mod->ras);
 
-  mod->user_b_freq->len = mod->ns;
+  mod->e_frq->user_b_freq->len = mod->ns;
 
   if(mod->whichmodel < 0)
     {
@@ -681,7 +871,12 @@ void Make_Model_Complete(t_mod *mod)
       Make_Custom_Model(mod);
       Translate_Custom_Mod_String(mod);
     }
-  if(mod->whichmodel == GTR)
+  
+  if(mod->io->datatype == NT && mod->whichmodel == GTR)
+    {
+      Make_Custom_Model(mod);
+    }
+  else if(mod->io->datatype == GENERIC)
     {
       Make_Custom_Model(mod);
     }
@@ -750,13 +945,27 @@ t_efrq *Make_Efrq(int ns)
   e_frq = (t_efrq *)mCalloc(1,sizeof(t_efrq));
 
   e_frq->pi               = (vect_dbl *)mCalloc(1,sizeof(vect_dbl));
-  e_frq->pi->v            = (phydbl *)mCalloc(ns,sizeof(phydbl));
   e_frq->pi->len          = ns;
 
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void **)&e_frq->pi->v,BYTE_ALIGN,(size_t)ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  e_frq->pi->v = _aligned_malloc(ns * sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+  e_frq->pi->v = (phydbl *)mCalloc(ns,sizeof(phydbl));
+#endif
+
+
   e_frq->pi_unscaled      = (vect_dbl *)mCalloc(1,sizeof(vect_dbl));
   e_frq->pi_unscaled->v   = (phydbl *)mCalloc(ns,sizeof(phydbl));
   e_frq->pi_unscaled->len = ns;
 
+  e_frq->user_b_freq      = (vect_dbl *)mCalloc(1,sizeof(vect_dbl));
+  Init_Vect_Dbl(0,e_frq->user_b_freq);
+  e_frq->user_b_freq->v   = (phydbl *)mCalloc(T_MAX_OPTION,sizeof(phydbl));
+
   return e_frq;
 }
 
@@ -790,7 +999,6 @@ t_rmat *Make_Rmat(int ns)
   r_mat->qmat->v                = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
   r_mat->qmat_buff->v           = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
 
-
   return(r_mat);
 }
 
@@ -814,14 +1022,16 @@ option *Make_Input()
   io->out_boot_stats_file               = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_stats_file                    = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_lk_file                       = (char *)mCalloc(T_MAX_FILE,sizeof(char));
+  io->weight_file                       = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_ps_file                       = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_summary_file                  = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_trace_file                    = (char *)mCalloc(T_MAX_FILE,sizeof(char));
+  io->out_json_trace_file               = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->nt_or_cd                          = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->run_id_string                     = (char *)mCalloc(T_MAX_OPTION,sizeof(char));
   io->clade_list_file                   = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->alphabet                          = (char **)mCalloc(T_MAX_ALPHABET,sizeof(char *));
-  For(i,T_MAX_ALPHABET) io->alphabet[i] = (char *)mCalloc(T_MAX_STATE,sizeof(char ));
+  for(i=0;i<T_MAX_ALPHABET;i++) io->alphabet[i] = (char *)mCalloc(T_MAX_STATE,sizeof(char ));
   io->treelist                          = (t_treelist *)mCalloc(1,sizeof(t_treelist));
   io->mcmc                              = (t_mcmc *)MCMC_Make_MCMC_Struct();
   io->rates                             = (t_rate *)RATES_Make_Rate_Struct(-1);
@@ -887,28 +1097,28 @@ triplet *Make_Triplet_Struct(t_mod *mod)
   triplet_struct->eigen_struct    = (eigen *)Make_Eigen_Struct(mod->ns);
   triplet_struct->mod             = mod;
 
-  For(k,mod->ras->n_catg)
+  for(k=0;k<mod->ras->n_catg;k++)
     {
       triplet_struct->core[k]                = (phydbl ***)mCalloc(mod->ns,sizeof(phydbl **));
-      For(i,mod->ns)
+      for(i=0;i<mod->ns;i++)
     {
       triplet_struct->core[k][i]         = (phydbl **)mCalloc(mod->ns,sizeof(phydbl *));
-      For(j,mod->ns)
+      for(j=0;j<mod->ns;j++)
         triplet_struct->core[k][i][j]    = (phydbl  *)mCalloc(mod->ns,sizeof(phydbl ));
     }
     }
 
-  For(i,mod->ns)
+  for(i=0;i<mod->ns;i++)
     {
       triplet_struct->p_one_site[i]          = (phydbl **)mCalloc(mod->ns,sizeof(phydbl *));
-      For(j,mod->ns)
+      for(j=0;j<mod->ns;j++)
     triplet_struct->p_one_site[i][j]     = (phydbl  *)mCalloc(mod->ns,sizeof(phydbl ));
     }
 
-  For(i,mod->ns)
+  for(i=0;i<mod->ns;i++)
     {
       triplet_struct->sum_p_one_site[i]      = (phydbl **)mCalloc(mod->ns,sizeof(phydbl *));
-      For(j,mod->ns)
+      for(j=0;j<mod->ns;j++)
     triplet_struct->sum_p_one_site[i][j] = (phydbl  *)mCalloc(mod->ns,sizeof(phydbl ));
     }
 
@@ -969,16 +1179,9 @@ void XML_Make_Node_Value(xml_node *n, char *val)
 
 xml_node *XML_Make_Node(char *name)
 {
-
   xml_node *new_node = (xml_node *)mCalloc(1,sizeof(xml_node));
-
-  if(name)
-    {
-      new_node->name = (char *)mCalloc(strlen(name)+1,sizeof(char));
-    }
-
+  if(name) new_node->name = (char *)mCalloc(strlen(name)+1,sizeof(char));
   new_node->ds = (t_ds *)mCalloc(1,sizeof(t_ds));
-
   return new_node;
 }
 
@@ -994,18 +1197,38 @@ void Make_Best_Spr(t_tree *tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-void Make_Spr_List(t_tree *tree)
+void Make_Spr_List_One_Edge(t_tree *tree)
 {
   int i;
 
-  tree->size_spr_list = 2*tree->n_otu-3;
+  tree->size_spr_list_one_edge = 2*tree->n_otu-3;
 
-  tree->spr_list = (t_spr **)mCalloc(2*tree->n_otu-2,sizeof(t_spr *));
+  tree->spr_list_one_edge = (t_spr **)mCalloc(2*tree->n_otu-2,sizeof(t_spr *));
 
-  For(i,2*tree->n_otu-2)
+  for(i=0;i<2*tree->n_otu-2;++i)
     {
-      tree->spr_list[i] = Make_One_Spr(tree);
-      Init_One_Spr(tree->spr_list[i]);
+      tree->spr_list_one_edge[i] = Make_One_Spr(tree);
+      Init_One_Spr(tree->spr_list_one_edge[i]);
+    }
+
+  tree->perform_spr_right_away = NO;
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Make_Spr_List_All_Edge(t_tree *tree)
+{
+  int i;
+
+  tree->size_spr_list_all_edge = 2*tree->n_otu-3;
+
+  tree->spr_list_all_edge = (t_spr **)mCalloc(2*tree->n_otu-2,sizeof(t_spr *));
+
+  for(i=0;i<2*tree->n_otu-2;++i)
+    {
+      tree->spr_list_all_edge[i] = Make_One_Spr(tree);
+      Init_One_Spr(tree->spr_list_all_edge[i]);
     }
 
   tree->perform_spr_right_away = NO;
@@ -1037,7 +1260,8 @@ t_rate *RATES_Make_Rate_Struct(int n_otu)
       rates->is_allocated         = YES;
       rates->nd_r                 = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->br_r                 = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->buff_r               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      rates->buff_br_r            = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      rates->buff_nd_r            = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->true_r               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->nd_t                 = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->buff_t               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
@@ -1086,11 +1310,12 @@ t_rate *RATES_Make_Rate_Struct(int n_otu)
       rates->survival_rank        = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->survival_dur         = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->calib_prob           = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->curr_nd_for_cal      = (int *)mCalloc(2*n_otu-1,sizeof(int));
       rates->t_prior_min_ori      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->t_prior_max_ori      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->times_partial_proba  = (phydbl *)mCalloc(n_otu*n_otu,sizeof(phydbl));
       rates->numb_calib_chosen    = (int *)mCalloc(n_otu*n_otu,sizeof(phydbl));
+      rates->a_cal                = (t_cal **)mCalloc(n_otu-1,sizeof(t_cal *));
+      rates->model_name           = (char *)mCalloc(T_MAX_NAME,sizeof(char));
     }
 
   return rates;
@@ -1099,22 +1324,55 @@ t_rate *RATES_Make_Rate_Struct(int n_otu)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-t_cal *Make_Calib(int n_otu)
+t_cal *Make_Calibration()
 {
   t_cal *calib;
-  int i;
-  i = 0;
-  calib                        = (t_cal *)mCalloc(1, sizeof(t_cal));
-  calib -> proba               = (phydbl *)mCalloc(2 * n_otu, sizeof(phydbl));
-  calib -> all_applies_to      = (t_node **)mCalloc(2 * n_otu - 1, sizeof(t_node *));
-  For(i, 2 * n_otu - 1)
-  calib -> all_applies_to[i]   = (t_node *)mCalloc(1, sizeof(t_node));
+  calib = (t_cal *)mCalloc(1, sizeof(t_cal));
   return(calib);
 }
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+t_clad *Make_Clade()
+{
+  t_clad *clade;
+  clade = (t_clad *)mCalloc(1, sizeof(t_clad));
+  return(clade);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+
+t_node **Make_Target_Tip(int n)
+{
+  t_node **this;
+  this = (t_node **)mCalloc(n,sizeof(t_node *));
+  return(this);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Make_All_Calibration(t_tree *tree)
+{
+  int i;
+  t_cal **all_cal;
+
+  assert(tree->rates);
+
+  all_cal = (t_cal **)mCalloc(2*tree->n_otu-1,sizeof(t_cal *));
+
+  For(i,2*tree->n_otu-1) all_cal[i] = Make_Calibration();
+
+  tree->rates->a_cal = all_cal;
+
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 void Make_Rmat_Weight(t_tree *mixt_tree)
 {
   t_tree *tree, *buff_tree;
@@ -1149,18 +1407,22 @@ void Make_Rmat_Weight(t_tree *mixt_tree)
     }
   while(tree);
 
-
+  
   tree = mixt_tree->next;
   tree->mod->r_mat_weight = (scalar_dbl *)mCalloc(1,sizeof(scalar_dbl));
   Init_Scalar_Dbl(tree->mod->r_mat_weight);
   tree->mod->r_mat_weight->v = 1.0;
   curr_weight = tree->mod->r_mat_weight;
 
+
   buff_tree = tree = mixt_tree;
   do // For each mixt_tree
     {
-      if(tree->is_mixt_tree == YES) tree = tree->next;
-      
+      if(tree->is_mixt_tree == YES) 
+        {
+          tree = tree->next;
+        }
+
       buff_tree = mixt_tree->next;
       do
         {
@@ -1232,7 +1494,6 @@ void Make_Efrq_Weight(t_tree *mixt_tree)
   while(tree);
 
 
-
   tree = mixt_tree->next;
   tree->mod->e_frq_weight = (scalar_dbl *)mCalloc(1,sizeof(scalar_dbl));
   Init_Scalar_Dbl(tree->mod->e_frq_weight);
@@ -1316,7 +1577,7 @@ void GEO_Make_Geo_Complete(int ldscape_sz, int n_dim, int n_tax, t_geo *t)
 
   // Locations
   t->coord_loc = (t_geo_coord **)mCalloc(ldscape_sz,sizeof(t_geo_coord *));
-  For(i,ldscape_sz) t->coord_loc[i] = GEO_Make_Geo_Coord(n_dim);
+  for(i=0;i<ldscape_sz;i++) t->coord_loc[i] = GEO_Make_Geo_Coord(n_dim);
 }
 
 //////////////////////////////////////////////////////////////
@@ -1397,13 +1658,69 @@ t_poly *Make_Poly(int n)
   int i;
   p = (t_poly *)mCalloc(1,sizeof(t_poly));
   p->poly_vert = (t_geo_coord **)mCalloc(n,sizeof(t_geo_coord *));
-  For(i,n) p->poly_vert[i] = GEO_Make_Geo_Coord(2);
+  for(i=0;i<n;i++) p->poly_vert[i] = GEO_Make_Geo_Coord(2);
   return(p);
 }
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
+
+t_sarea *Make_Sarea(int n_poly)
+{
+  t_sarea *s;
+  s = (t_sarea *)mCalloc(1,sizeof(t_sarea ));
+  s->a_poly = (t_poly **)mCalloc(n_poly,sizeof(t_poly *));
+  return(s);
+}
+
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
+
+t_ll *Make_Linked_List()
+{
+  t_ll *list;
+
+  // Create list if non-existing and add
+  list = (t_ll *)mCalloc(1,sizeof(t_ll));
+
+  return list;
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+/* Matrices used in transfer bootstrap computation (tbe.c) */
+void Alloc_TBE_Matrices(int n_otu, short unsigned*** i_matrix, short unsigned*** c_matrix,short unsigned*** hamming, short unsigned** min_dist, short unsigned**  min_dist_edge, int** cluster_sizes){
+  int i;
+  int nb_edges = 2*n_otu-3;
+  (*min_dist) = (short unsigned*) malloc(nb_edges*sizeof(short unsigned)); /* array of min Hamming distances */
+  (*min_dist_edge) = (short unsigned*) malloc(nb_edges*sizeof(short unsigned)); /* array of edge ids corresponding to min Hamming distances */
+  (*cluster_sizes) = (int*) malloc(nb_edges*sizeof(int)); /* array of sizes of clusters associated to each branch (in the post order traversal) */
+  (*c_matrix) = (short unsigned**) malloc(nb_edges*sizeof(short unsigned*)); /* matrix of cardinals of complements */
+  (*i_matrix) = (short unsigned**) malloc(nb_edges*sizeof(short unsigned*)); /* matrix of cardinals of intersections */
+  (*hamming) = (short unsigned**) malloc(nb_edges*sizeof(short unsigned*)); /* matrix of Hamming distances */
+  for (i=0; i<nb_edges; i++){
+    (*c_matrix)[i] = (short unsigned*) malloc(nb_edges*sizeof(short unsigned));
+    (*i_matrix)[i] = (short unsigned*) malloc(nb_edges*sizeof(short unsigned));
+    (*hamming)[i] = (short unsigned*) malloc(nb_edges*sizeof(short unsigned));
+    (*min_dist)[i] = n_otu; /* initialization to the nb of taxa */
+    (*cluster_sizes)[i] = 0;
+  }
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Make_MutMap(t_tree *tree)
+{
+  // (# of edges) X (# of sites) X (# mutation types: A<->C A<->G A<->T C<->G C<->T G<->T)
+  do
+    {
+      tree->mutmap = (int *)mCalloc((2*tree->n_otu-3)*(tree->n_pattern)*6,sizeof(int));
+      tree = tree->next_mixt;
+    }
+  while(tree);
+}
+
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
