@@ -1,6 +1,6 @@
 /*
 
-PhyML:  a program that  computes maximum likelihood phyLOGenies from
+PhyML:  a program that  computes maximum likelihood phylogenies from
 DNA or AA homologous sequences.
 
 Copyright (C) Stephane Guindon. Oct 2003 onward.
@@ -16,7 +16,11 @@ the GNU public licence. See http://www.opensource.org for details.
 
 /*********************************************************/
 
-void Bootstrap_MPI(t_tree *tree)
+/* 
+   if tbe_bootstrap == 0 => Classical FBP (Felsenstein bootstrap proportions) 
+   else => TBE (Transfer bootstrap expectation)
+*/
+void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
 {
   int *site_num, n_site;
   int replicate,j,k;
@@ -33,9 +37,17 @@ void Bootstrap_MPI(t_tree *tree)
   int *score_par, *score_tot;
   char *bootStr, *t;
   
+  if(tree->is_mixt_tree == YES)
+    {
+      PhyML_Printf("\n== Bootstrap option not yet available for partition/mixture analysis.");
+      assert(FALSE);
+    }
+
   randomRecv = nbElem = bootRecv = 0;
 
-  tree->print_boot_val       = 1;
+  tree->print_boot_val       = !tbe_bootstrap;
+  tree->print_tbe_val        = tbe_bootstrap;
+  
   tree->print_alrt_val       = 0;
   boot_tree                  = NULL;
 
@@ -46,7 +58,7 @@ void Bootstrap_MPI(t_tree *tree)
   Get_Bip(tree->a_nodes[0],tree->a_nodes[0]->v[0],tree);
 
   n_site = 0;
-  For(j,tree->data->crunch_len) For(k,tree->data->wght[j])
+  for(j=0;j<tree->data->crunch_len;j++) For(k,tree->data->wght[j])
     {
       site_num[n_site] = j;
       n_site++;
@@ -56,6 +68,13 @@ void Bootstrap_MPI(t_tree *tree)
 
   if (Global_myRank == 0)
     PhyML_Printf("\n. Non parametric bootstrap analysis \n");
+
+  if(Global_numTask <= 1)
+    {
+      PhyML_Printf("\n. The number of CPUs used in the MPI bootstrap analysis should be");
+      PhyML_Printf("\n. strictly greater than 1.");
+      assert(FALSE);
+    }
   
   //number of bootstraps for each process
   if (tree->mod->bootstrap%Global_numTask != 0) 
@@ -64,7 +83,7 @@ void Bootstrap_MPI(t_tree *tree)
       tree->mod->bootstrap = nbRep * Global_numTask;
       if (Global_myRank == 0) {
         PhyML_Printf("\n. The number of replicates is not a multiple of %d CPUs.\n", Global_numTask);
-        PhyML_Printf("\n. Will run %d replicates analysis.\n", tree->mod->bootstrap);
+        PhyML_Printf("\n. Will run %d replicates.\n", tree->mod->bootstrap);
       }
     }
   else
@@ -89,63 +108,77 @@ void Bootstrap_MPI(t_tree *tree)
 
   For(replicate, nbRep)
     {
-      For(j,boot_data->crunch_len) boot_data->wght[j] = 0;
+      for(j=0;j<boot_data->crunch_len;j++) boot_data->wght[j] = 0;
 
       // Send random data to other process
-      if (Global_myRank == 0) {
-        // Compute number of data to send
-        if (tree->mod->bootstrap - randomRecv > Global_numTask)
-          nbElem = Global_numTask;
-        else
-          nbElem = tree->mod->bootstrap - randomRecv;
-
-        For(i,nbElem) {
-          For(j,boot_data->crunch_len) boot_data->wght[j] = 0;
-          init_len = 0;
-          // Create random data
-          For(j,boot_data->init_len)
+      if (Global_myRank == 0) 
+        {
+          // Compute number of data to send
+          if (tree->mod->bootstrap - randomRecv > Global_numTask)
+            nbElem = Global_numTask;
+          else
+            nbElem = tree->mod->bootstrap - randomRecv;
+          
+          for(i=0;i<nbElem;i++) 
             {
-              position = Rand_Int(0,(int)(tree->data->init_len-1.0));
-              boot_data->wght[site_num[position]] += 1;
-              init_len++;
-            }
-            
+              for(j=0;j<boot_data->crunch_len;j++) boot_data->wght[j] = 0;
 
-          if (init_len != tree->data->init_len) {
-            MPI_Finalize();
-            Warn_And_Exit("\n== Pb. when copying sequences...\n");
-          }
-          // Send random data to other process, not to current process
-          if (i < nbElem-1) {
-            MPI_Ssend (boot_data->wght, boot_data->crunch_len, MPI_INT, i+1, Global_myRank, MPI_COMM_WORLD);
+              init_len = 0;
+              // Create random data
+              for(j=0;j<boot_data->init_len;j++)
+                {
+                  position = Rand_Int(0,(int)(tree->data->init_len-1.0));
+                  boot_data->wght[site_num[position]] += 1;
+                  init_len++;
+                }
+              
+
+              if (init_len != tree->data->init_len) 
+                {
+                  PhyML_Printf("\n== thread: %d || init: %d %d here\n",Global_myRank,init_len,tree->data->init_len);
+                  fflush(NULL);
+                  Generic_Exit(__FILE__,__LINE__,__FUNCTION__);    
+                }
+              
+              // Send random data to other process, not to current process
+              if (i < nbElem-1)
+                {
+                  MPI_Ssend (boot_data->wght, boot_data->crunch_len, MPI_DOUBLE, i+1, Global_myRank, MPI_COMM_WORLD);
 #ifdef MPI_DEBUG
-fprintf (stderr, "\ntask %d, sending random to %d done\n", Global_myRank, i+1);
-fflush(stderr);
+                  fprintf (stderr, "\ntask %d, sending random to %d done\n", Global_myRank, i+1);
+                  fflush(stderr);
 #endif
-          }
-          randomRecv++;
+                }
+              randomRecv++;
+            }
         }
-      }
-      else {
-        MPI_Recv (boot_data->wght, boot_data->crunch_len, MPI_INT, 0, MPI_ANY_TAG, MPI_COMM_WORLD, &Stat);
+      else 
+        {
+          MPI_Recv(boot_data->wght, boot_data->crunch_len, MPI_DOUBLE, 0, MPI_ANY_TAG, MPI_COMM_WORLD, &Stat);
 #ifdef MPI_DEBUG
-fprintf (stderr, "\ntask %d, receiving random from task %d done\n", Global_myRank, Stat.MPI_SOURCE);
-fflush(stderr);
+          fprintf (stderr, "\ntask %d, receiving random from task %d done\n", Global_myRank, Stat.MPI_SOURCE);
+          fflush(stderr);
 #endif
-      }
+
+
+        }
 
       init_len = 0;
-      For(j,boot_data->crunch_len) init_len += boot_data->wght[j];
+      for(j=0;j<boot_data->crunch_len;j++) init_len += boot_data->wght[j];
 
-      if(init_len != tree->data->init_len) {
-        MPI_Finalize();
-        Warn_And_Exit("\n== Pb when copying sequences\n");
-      }
+
+      if(init_len != tree->data->init_len) 
+        {
+          printf("\n== thread: %d init: %d %d\n",Global_myRank,init_len,tree->data->init_len);
+          Generic_Exit(__FILE__,__LINE__,__FUNCTION__);    
+        }
+      
 
       (tree->mod->io->datatype == NT)?
         (Get_Base_Freqs(boot_data)):
         (Get_AA_Freqs(boot_data));
 
+
       if(tree->io->random_boot_seq_order) Randomize_Sequence_Order(boot_data);
 
       Set_D_States(boot_data,tree->io->datatype,tree->io->state_len);
@@ -171,9 +204,9 @@ fflush(stderr);
 	      }
 	    case NEXUS:
 	      {
-		PhyML_Printf("\n. Unfortunately, PhyML cannot read NEXUS files and perform a bootstrap analysis."); 
-		PhyML_Printf("\n. Please use the PHYLIP format.."); 
-		PhyML_Printf("\n. Err in file %s at line %d\n",__FILE__,__LINE__);
+		PhyML_Printf("\n== Unfortunately, PhyML cannot read NEXUS files and perform a bootstrap analysis."); 
+		PhyML_Printf("\n== Please use the PHYLIP format."); 
+		PhyML_Printf("\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
 		Warn_And_Exit("");
 		break;
 	      }
@@ -198,7 +231,7 @@ fflush(stderr);
       boot_tree->mod                = boot_mod;
       boot_tree->io                 = tree->io;
       boot_tree->data               = boot_data;
-      boot_tree->mod->s_opt->print  = 0;
+      boot_tree->verbose            = VL0;
       boot_tree->n_pattern          = boot_tree->data->crunch_len;
       boot_tree->io->print_site_lnl = 0;
       boot_tree->io->print_trace    = 0;
@@ -214,28 +247,24 @@ fflush(stderr);
       Fill_Dir_Table(boot_tree);
       Update_Dirs(boot_tree);
 
-      if(tree->mod->s_opt->greedy) Init_P_Lk_Tips_Double(boot_tree);
-      else                         Init_P_Lk_Tips_Int(boot_tree);
+      if(tree->mod->s_opt->greedy) Init_Partial_Lk_Tips_Double(boot_tree);
+      else                         Init_Partial_Lk_Tips_Int(boot_tree);
       Init_Ui_Tips(boot_tree);
-      Init_P_Pars_Tips(boot_tree);
+      Init_Partial_Pars_Tips(boot_tree);
       Br_Len_Not_Involving_Invar(boot_tree);
       
+      Switch_Eigen(YES,boot_tree->mod);                 
+      Lk(NULL,boot_tree);
+      Switch_Eigen(NO,boot_tree->mod);
+
       if(boot_tree->mod->s_opt->opt_topo)
         {
-          if(boot_tree->mod->s_opt->topo_search == NNI_MOVE) 
-            {
-              Simu_Loop(boot_tree);
-            }
-          else if((boot_tree->mod->s_opt->topo_search == SPR_MOVE) ||
-                  (boot_tree->mod->s_opt->topo_search == BEST_OF_NNI_AND_SPR))
-            {
-              Speed_Spr_Loop(boot_tree);
-            }
+          Global_Spr_Search(boot_tree);
         }
       else
         {
           if(boot_tree->mod->s_opt->opt_subst_param || boot_tree->mod->s_opt->opt_bl)
-            Round_Optimize(boot_tree,boot_tree->data,ROUND_MAX);
+            Round_Optimize(boot_tree,ROUND_MAX);
           else
             Lk(NULL,boot_tree);
         }
@@ -250,7 +279,11 @@ fflush(stderr);
               boot_tree->a_nodes[0]->v[0],
               boot_tree);
 
-      Compare_Bip(tree,boot_tree,NO);
+      if(!tbe_bootstrap){
+	Compare_Bip(tree,boot_tree,NO);
+      }else{
+	Compare_Bip_Distance(tree, boot_tree);
+      }
       
       Br_Len_Involving_Invar(boot_tree);
 
@@ -350,7 +383,7 @@ fflush(stdout);
     if(((bootRecv)%tree->io->boot_prog_every)) PhyML_Printf("] %4d/%4d\n ",bootRecv,tree->mod->bootstrap);
 
 PhyML_Printf("\n\n. Exiting bootstrap function normally."); fflush(NULL);
-  tree->lock_topo = 1; /* TopoLOGy should not be modified afterwards */
+tree->lock_topo = 1; /* Topology should not be modified afterwards */
 
   if(tree->io->print_boot_trees)
     {
@@ -358,7 +391,7 @@ PhyML_Printf("\n\n. Exiting bootstrap function normally."); fflush(NULL);
       fclose(tree->io->fp_out_boot_stats);
     }
 
-  Free_Cseq(boot_data);
+  Free_Calign(boot_data);
   Free(site_num);
 }
 
@@ -499,7 +532,7 @@ void Print_Fp_Out_Lines_MPI(t_tree *tree, option *io, int n_data_set, char *boot
     {
       int i,j;
 
-      For(i,4)
+      for(i=0;i<4;i++)
         {
           if (i!=0) {
             /*format*/
@@ -512,7 +545,7 @@ void Print_Fp_Out_Lines_MPI(t_tree *tree, option *io, int n_data_set, char *boot
             snprintf(tmp, T_MAX_LINE, "             \t                                      \t");
             strncat (s, tmp, T_MAX_LINE);
           }
-          For(j,4) {
+          for(j=0;j<4;j++) {
             snprintf(tmp, T_MAX_LINE, "%8.5f  ",tree->mod->r_mat->qmat->v[i*4+j]);
             strncat (s, tmp, T_MAX_LINE);
           }
