@@ -1,7 +1,7 @@
 /*
 
-PHYML :  a program that  computes maximum likelihood  phyLOGenies from
-DNA or AA homoLOGous sequences
+PHYML :  a program that  computes maximum likelihood  phylogenies from
+DNA or AA homologous sequences
 
 Copyright (C) Stephane Guindon. Oct 2003 onward
 
@@ -21,18 +21,25 @@ int Pars(t_edge *b, t_tree *tree)
 {
   int site,n_patterns;
 
+  if(tree->is_mixt_tree == YES) 
+    {
+      MIXT_Pars(b,tree);
+      return tree->c_pars;
+    }
+
   n_patterns = tree->n_pattern;
 
-  if(!b)
+  
+  if(b == NULL)
     {
       Post_Order_Pars(tree->a_nodes[0],tree->a_nodes[0]->v[0],tree);
       if(tree->both_sides == YES) Pre_Order_Pars(tree->a_nodes[0],tree->a_nodes[0]->v[0],tree);
     }
 
-  if(!b) b = tree->a_nodes[0]->b[0];
+  if(b == NULL) b = tree->a_nodes[0]->b[0];
 
   tree->c_pars = 0;
-  For(site,n_patterns)
+  for(site=0;site<n_patterns;site++)
     {
       tree->site_pars[site] = 0;
       tree->curr_site       = site;
@@ -41,11 +48,6 @@ int Pars(t_edge *b, t_tree *tree)
       /* printf("\n. site %d pars: %d",site,tree->c_pars); */
     }
 
-  if(tree->is_mixt_tree)
-    {
-      MIXT_Pars(b,tree);
-    }
-
   return tree->c_pars;
 }
 
@@ -60,7 +62,7 @@ void Post_Order_Pars(t_node *a, t_node *d, t_tree *tree)
   if(d->tax) return;
   else
     {
-      For(i,3)
+      for(i=0;i<3;i++)
     {
       if(d->v[i] != a)
         Post_Order_Pars(d,d->v[i],tree);
@@ -79,22 +81,22 @@ void Pre_Order_Pars(t_node *a, t_node *d, t_tree *tree)
   if(d->tax) return;
   else
     {
-      For(i,3)
-    {
-      if(d->v[i] != a)
+      for(i=0;i<3;i++)
         {
-          Get_All_Partial_Pars(tree,d->b[i],d->v[i],d);
-          Pre_Order_Pars(d,d->v[i],tree);
+          if(d->v[i] != a)
+            {
+              Get_All_Partial_Pars(tree,d->b[i],d->v[i],d);
+              Pre_Order_Pars(d,d->v[i],tree);
+            }
         }
     }
-    }
 }
 
 /*********************************************************/
 
 void Get_All_Partial_Pars(t_tree *tree, t_edge *b_fcus, t_node *a, t_node *d)
 {
-  Update_P_Pars(tree,b_fcus,d);
+  Update_Partial_Pars(tree,b_fcus,d);
 }
 
 /*********************************************************/
@@ -106,7 +108,7 @@ void Site_Pars(t_tree *tree)
 
 /*********************************************************/
 
-void Init_P_Pars_Tips(t_tree *tree)
+void Init_Partial_Pars_Tips(t_tree *tree)
 {
   int curr_site,i,j;
   short int *state_v;
@@ -116,45 +118,43 @@ void Init_P_Pars_Tips(t_tree *tree)
 
   state_v = (short int *)mCalloc(tree->mod->ns,sizeof(short int));
 
-
-
-  For(curr_site,tree->data->crunch_len)
+  for(curr_site=0;curr_site<tree->data->crunch_len;curr_site++)
     {
-      For(i,tree->n_otu)
-    {
-      if(tree->a_nodes[i]->b[0]->rght->tax != 1)
+      for(i=0;i<tree->n_otu;i++)
         {
-	      PhyML_Printf("\n== Err. in file %s at line %d\n",__FILE__,__LINE__);
-          Exit("\n");
+          if(tree->a_nodes[i]->b[0]->rght->tax != 1)
+            {
+	      PhyML_Fprintf(stderr,"\n. Err. in file %s at line %d\n",__FILE__,__LINE__);
+              Exit("\n");
+            }
+          
+          if(tree->io->datatype == NT)
+            {
+              Init_Tips_At_One_Site_Nucleotides_Int(tree->a_nodes[i]->c_seq->state[curr_site],
+                                                    0,
+                                                    state_v);
+              for(j=0;j<tree->mod->ns;j++) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] = MAX_PARS;
+              for(j=0;j<tree->mod->ns;j++) if(state_v[j] > 0.5) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] =  0;
+            }
+          else if(tree->io->datatype == AA)
+            {
+              Init_Tips_At_One_Site_AA_Int(tree->a_nodes[i]->c_seq->state[curr_site],
+                                           0,
+                                           state_v);
+              for(j=0;j<tree->mod->ns;j++) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] = MAX_PARS;
+              for(j=0;j<tree->mod->ns;j++) if(state_v[j] > 0.5) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] =  0;
+            }
+          else if(tree->io->datatype == GENERIC)
+            {
+              Init_Tips_At_One_Site_Generic_Int(tree->a_nodes[i]->c_seq->state+curr_site*tree->mod->io->state_len,
+                                                tree->mod->ns,
+                                                tree->mod->io->state_len,
+                                                0,
+                                                state_v);
+              for(j=0;j<tree->mod->ns;j++) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] = MAX_PARS;
+              for(j=0;j<tree->mod->ns;j++) if(state_v[j] > 0.5) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] =  0;
+            }
         }
-
-      if(tree->io->datatype == NT)
-        {
-          Init_Tips_At_One_Site_Nucleotides_Int(tree->a_nodes[i]->c_seq->state[curr_site],
-                            0,
-                            state_v);
-          For(j,tree->mod->ns) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] = MAX_PARS;
-          For(j,tree->mod->ns) if(state_v[j] > 0.5) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] =  0;
-        }
-      else if(tree->io->datatype == AA)
-        {
-          Init_Tips_At_One_Site_AA_Int(tree->a_nodes[i]->c_seq->state[curr_site],
-                       0,
-                       state_v);
-          For(j,tree->mod->ns) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] = MAX_PARS;
-          For(j,tree->mod->ns) if(state_v[j] > 0.5) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] =  0;
-        }
-      else if(tree->io->datatype == GENERIC)
-        {
-          Init_Tips_At_One_Site_Generic_Int(tree->a_nodes[i]->c_seq->state+curr_site*tree->mod->io->state_len,
-                        tree->mod->ns,
-                        tree->mod->io->state_len,
-                        0,
-                        state_v);
-          For(j,tree->mod->ns) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] = MAX_PARS;
-          For(j,tree->mod->ns) if(state_v[j] > 0.5) tree->a_nodes[i]->b[0]->p_pars_r[curr_site*dim1+j] =  0;
-        }
-    }
     }
   Free(state_v);
 }
@@ -168,15 +168,15 @@ void Init_Ui_Tips(t_tree *tree)
 
   state_v = (short int *)mCalloc(tree->mod->ns,sizeof(short int));
 
-  For(curr_site,tree->data->crunch_len)
+  for(curr_site=0;curr_site<tree->data->crunch_len;curr_site++)
     {
-      For(i,tree->n_otu)
+      for(i=0;i<tree->n_otu;++i)
         {
           if(tree->io->datatype == NT)
             {
               if(tree->a_nodes[i]->b[0]->rght->tax != 1)
                 {
-                  PhyML_Printf("\n== Err in file %s at line %d\n",__FILE__,__LINE__);
+                  PhyML_Fprintf(stderr,"\n. Err. in file %s at line %d\n",__FILE__,__LINE__);
                   Exit("\n");
                 }
               
@@ -187,7 +187,7 @@ void Init_Ui_Tips(t_tree *tree)
               /* 					    0, */
               /* 					    state_v);	       */
               tree->a_nodes[i]->b[0]->ui_r[curr_site] = 0;
-              For(j,tree->mod->ns) tree->a_nodes[i]->b[0]->ui_r[curr_site] += (unsigned int)(state_v[j] * POW(2,j));
+              for(j=0;j<tree->mod->ns;j++) tree->a_nodes[i]->b[0]->ui_r[curr_site] += (int)(state_v[j] * POW(2,j));
             }
           else if(tree->io->datatype == AA)
             {
@@ -198,7 +198,7 @@ void Init_Ui_Tips(t_tree *tree)
               /* 				   0, */
               /* 				   state_v); */
               tree->a_nodes[i]->b[0]->ui_r[curr_site] = 0;
-              For(j,tree->mod->ns) tree->a_nodes[i]->b[0]->ui_r[curr_site] += (unsigned int)(state_v[j] * POW(2,j));
+              for(j=0;j<tree->mod->ns;j++) tree->a_nodes[i]->b[0]->ui_r[curr_site] += (int)(state_v[j] * POW(2,j));
             }
           else if(tree->io->datatype == GENERIC)
             {
@@ -213,7 +213,7 @@ void Init_Ui_Tips(t_tree *tree)
               /* 					0, */
               /* 					state_v); */
               tree->a_nodes[i]->b[0]->ui_r[curr_site] = 0;
-              For(j,tree->mod->ns) tree->a_nodes[i]->b[0]->ui_r[curr_site] += (unsigned int)(state_v[j] * POW(2,j));
+              for(j=0;j<tree->mod->ns;j++) tree->a_nodes[i]->b[0]->ui_r[curr_site] += (int)(state_v[j] * POW(2,j));
             }
         }
     }
@@ -221,7 +221,7 @@ void Init_Ui_Tips(t_tree *tree)
   
   For(br,2*tree->n_otu-3)
     {
-      For(curr_site,tree->data->crunch_len)
+      for(curr_site=0;curr_site<tree->data->crunch_len;curr_site++)
         {
           tree->a_edges[br]->pars_r[curr_site] = 0;
           tree->a_edges[br]->pars_l[curr_site] = 0;
@@ -234,7 +234,7 @@ void Init_Ui_Tips(t_tree *tree)
 
 /*********************************************************/
 
-void Update_P_Pars(t_tree *tree, t_edge *b_fcus, t_node *n)
+void Update_Partial_Pars(t_tree *tree, t_edge *b_fcus, t_node *n)
 {
 /*
            |
@@ -246,15 +246,22 @@ void Update_P_Pars(t_tree *tree, t_edge *b_fcus, t_node *n)
         /     \
 */
 
-  int i,j;
-  int site;
-  unsigned int *ui, *ui_v1, *ui_v2;
+  unsigned int i,j;
+  unsigned int site;
+  int *ui, *ui_v1, *ui_v2;
   int *p_pars_v1, *p_pars_v2, *p_pars;
   int *pars, *pars_v1, *pars_v2;
-  int n_patterns;
   int min_v1,min_v2;
   int v;
-  int dim1;
+  
+  const unsigned int ns = tree->mod->ns;
+  const unsigned int n_patterns = tree->n_pattern;
+
+  if(tree->is_mixt_tree)
+    {
+      MIXT_Update_Partial_Pars(tree,b_fcus,n);
+      return;
+    }
 
   if((tree->io->do_alias_subpatt == YES) &&
      (tree->update_alias_subpatt == YES))
@@ -262,128 +269,124 @@ void Update_P_Pars(t_tree *tree, t_edge *b_fcus, t_node *n)
 
   if(n->tax) return;
 
-  dim1 = tree->mod->ns;
   ui = ui_v1 = ui_v2 = NULL;
   p_pars = p_pars_v1 = p_pars_v2 = NULL;
   pars = pars_v1 = pars_v2 = NULL;
 
-  n_patterns = tree->n_pattern;
-
-
+    
   if(n == b_fcus->left)
     {
       ui = b_fcus->ui_l;
-
+      
       pars = b_fcus->pars_l;
       p_pars = b_fcus->p_pars_l;
-
+      
       ui_v1 =
-      (n == n->b[b_fcus->l_v1]->left)?
-      (n->b[b_fcus->l_v1]->ui_r):
-      (n->b[b_fcus->l_v1]->ui_l);
-
+        (n == n->b[b_fcus->l_v1]->left)?
+        (n->b[b_fcus->l_v1]->ui_r):
+        (n->b[b_fcus->l_v1]->ui_l);
+      
       ui_v2 =
-      (n == n->b[b_fcus->l_v2]->left)?
-      (n->b[b_fcus->l_v2]->ui_r):
-      (n->b[b_fcus->l_v2]->ui_l);
-
+        (n == n->b[b_fcus->l_v2]->left)?
+        (n->b[b_fcus->l_v2]->ui_r):
+        (n->b[b_fcus->l_v2]->ui_l);
+      
       p_pars_v1 =
-      (n == n->b[b_fcus->l_v1]->left)?
-      (n->b[b_fcus->l_v1]->p_pars_r):
-      (n->b[b_fcus->l_v1]->p_pars_l);
-
+        (n == n->b[b_fcus->l_v1]->left)?
+        (n->b[b_fcus->l_v1]->p_pars_r):
+        (n->b[b_fcus->l_v1]->p_pars_l);
+      
       p_pars_v2 =
-      (n == n->b[b_fcus->l_v2]->left)?
-      (n->b[b_fcus->l_v2]->p_pars_r):
-      (n->b[b_fcus->l_v2]->p_pars_l);
-
+        (n == n->b[b_fcus->l_v2]->left)?
+        (n->b[b_fcus->l_v2]->p_pars_r):
+        (n->b[b_fcus->l_v2]->p_pars_l);
+      
       pars_v1 =
-      (n == n->b[b_fcus->l_v1]->left)?
-      (n->b[b_fcus->l_v1]->pars_r):
-      (n->b[b_fcus->l_v1]->pars_l);
-
+        (n == n->b[b_fcus->l_v1]->left)?
+        (n->b[b_fcus->l_v1]->pars_r):
+        (n->b[b_fcus->l_v1]->pars_l);
+      
       pars_v2 =
-      (n == n->b[b_fcus->l_v2]->left)?
-      (n->b[b_fcus->l_v2]->pars_r):
-      (n->b[b_fcus->l_v2]->pars_l);
+        (n == n->b[b_fcus->l_v2]->left)?
+        (n->b[b_fcus->l_v2]->pars_r):
+        (n->b[b_fcus->l_v2]->pars_l);
     }
   else
     {
       ui = b_fcus->ui_r;
-
+      
       pars = b_fcus->pars_r;
       p_pars = b_fcus->p_pars_r;
-
+      
       ui_v1 =
-      (n == n->b[b_fcus->r_v1]->left)?
-      (n->b[b_fcus->r_v1]->ui_r):
-      (n->b[b_fcus->r_v1]->ui_l);
-
+        (n == n->b[b_fcus->r_v1]->left)?
+        (n->b[b_fcus->r_v1]->ui_r):
+        (n->b[b_fcus->r_v1]->ui_l);
+      
       ui_v2 =
-      (n == n->b[b_fcus->r_v2]->left)?
-      (n->b[b_fcus->r_v2]->ui_r):
-      (n->b[b_fcus->r_v2]->ui_l);
-
+        (n == n->b[b_fcus->r_v2]->left)?
+        (n->b[b_fcus->r_v2]->ui_r):
+        (n->b[b_fcus->r_v2]->ui_l);
+      
       p_pars_v1 =
-      (n == n->b[b_fcus->r_v1]->left)?
-      (n->b[b_fcus->r_v1]->p_pars_r):
-      (n->b[b_fcus->r_v1]->p_pars_l);
-
+        (n == n->b[b_fcus->r_v1]->left)?
+        (n->b[b_fcus->r_v1]->p_pars_r):
+        (n->b[b_fcus->r_v1]->p_pars_l);
+      
       p_pars_v2 =
-      (n == n->b[b_fcus->r_v2]->left)?
-      (n->b[b_fcus->r_v2]->p_pars_r):
-      (n->b[b_fcus->r_v2]->p_pars_l);
-
+        (n == n->b[b_fcus->r_v2]->left)?
+        (n->b[b_fcus->r_v2]->p_pars_r):
+        (n->b[b_fcus->r_v2]->p_pars_l);
+      
       pars_v1 =
-      (n == n->b[b_fcus->r_v1]->left)?
-      (n->b[b_fcus->r_v1]->pars_r):
-      (n->b[b_fcus->r_v1]->pars_l);
-
+        (n == n->b[b_fcus->r_v1]->left)?
+        (n->b[b_fcus->r_v1]->pars_r):
+        (n->b[b_fcus->r_v1]->pars_l);
+      
       pars_v2 =
-      (n == n->b[b_fcus->r_v2]->left)?
-      (n->b[b_fcus->r_v2]->pars_r):
-      (n->b[b_fcus->r_v2]->pars_l);
+        (n == n->b[b_fcus->r_v2]->left)?
+        (n->b[b_fcus->r_v2]->pars_r):
+        (n->b[b_fcus->r_v2]->pars_l);
     }
-
-
+  
+  
   if(tree->mod->s_opt->general_pars)
     {
-      For(site,n_patterns)
-    {
-      For(i,tree->mod->ns)
+      for(site=0;site<n_patterns;++site)
         {
-          min_v1 = MAX_PARS;
-          For(j,tree->mod->ns)
-        {
-          v = p_pars_v1[site*dim1+j] + tree->step_mat[i*tree->mod->ns+j];
-          if(v < min_v1) min_v1 = v;
+          for(i=0;i<ns;++i)
+            {
+              min_v1 = MAX_PARS;
+              for(j=0;j<ns;++j)
+                {
+                  v = p_pars_v1[site*ns+j] + tree->step_mat[i*ns+j];
+                  if(v < min_v1) min_v1 = v;
+                }
+              
+              min_v2 = MAX_PARS;
+              for(j=0;j<ns;++j)
+                {
+                  v = p_pars_v2[site*ns+j] + tree->step_mat[i*ns+j];
+                  if(v < min_v2) min_v2 = v;
+                }
+              p_pars[site*ns+i] = min_v1 + min_v2;
+            }
         }
-
-          min_v2 = MAX_PARS;
-          For(j,tree->mod->ns)
-        {
-          v = p_pars_v2[site*dim1+j] + tree->step_mat[i*tree->mod->ns+j];
-          if(v < min_v2) min_v2 = v;
-        }
-          p_pars[site*dim1+i] = min_v1 + min_v2;
-        }
-    }
     }
   else
     {
-      For(site,n_patterns)
-    {
-      pars[site] = pars_v1[site] + pars_v2[site];
-
-      ui[site] = ui_v1[site] & ui_v2[site];
-
-      if(!ui[site])
+      for(site=0;site<n_patterns;++site)
         {
-          pars[site]++;
-          ui[site] = ui_v1[site] | ui_v2[site];
+          pars[site] = pars_v1[site] + pars_v2[site];
+          
+          ui[site] = ui_v1[site] & ui_v2[site];
+          
+          if(!ui[site])
+            {
+              pars[site]++;
+              ui[site] = ui_v1[site] | ui_v2[site];
+            }          
         }
-
-    }
     }
 }
 
@@ -392,43 +395,42 @@ void Update_P_Pars(t_tree *tree, t_edge *b_fcus, t_node *n)
 int Pars_Core(t_edge *b, t_tree *tree)
 {
   int site;
-  int i,j;
+  unsigned int i,j;
   int site_pars;
   int min_l,min_r;
   int v;
-  int dim1;
-
-  dim1 = tree->mod->ns;
+  
+  const unsigned int ns = tree->mod->ns;
   site = tree->curr_site;
   site_pars = MAX_PARS;
 
   if(tree->mod->s_opt->general_pars)
     {
-      For(i,tree->mod->ns)
-    {
-      min_l = MAX_PARS;
-      For(j,tree->mod->ns)
+      for(i=0;i<ns;++i)
         {
-          v = b->p_pars_l[site*dim1+j] + tree->step_mat[i*tree->mod->ns+j];
-          if(v < min_l) min_l = v;
+          min_l = MAX_PARS;
+          for(j=0;j<ns;++j)
+            {
+              v = b->p_pars_l[site*ns+j] + tree->step_mat[i*ns+j];
+              if(v < min_l) min_l = v;
+            }
+          
+          min_r = MAX_PARS;
+          for(j=0;j<ns;++j)
+            {
+              v = b->p_pars_r[site*ns+j] + tree->step_mat[i*ns+j];
+              if(v < min_r) min_r = v;
+            }
+          
+          if((min_l + min_r) < site_pars) site_pars = min_l + min_r;
         }
-
-      min_r = MAX_PARS;
-      For(j,tree->mod->ns)
-        {
-          v = b->p_pars_r[site*dim1+j] + tree->step_mat[i*tree->mod->ns+j];
-          if(v < min_r) min_r = v;
-        }
-
-      if((min_l + min_r) < site_pars) site_pars = min_l + min_r;
-    }
     }
   else
     {
       site_pars = b->pars_l[site] + b->pars_r[site];
       if(!(b->ui_l[site] & b->ui_r[site])) site_pars++;
     }
-
+  
   return site_pars;
 }
 
@@ -447,7 +449,7 @@ int One_Pars_Step(t_edge *b,t_tree *tree)
   Set_Both_Sides(YES,tree);
   Pars(NULL,tree);
 
-  For(site,tree->n_pattern)
+  for(site=0;site<tree->n_pattern;site++)
     {
       if(!(b->ui_l[site] & b->ui_r[site])) break;
     }
@@ -469,7 +471,7 @@ int Pars_At_Given_Edge(t_edge *b, t_tree *tree)
   n_patterns = tree->n_pattern;
 
   tree->c_pars = .0;
-  For(site,n_patterns)
+  for(site=0;site<n_patterns;site++)
     {
       tree->site_pars[site] = 0;
       tree->curr_site = site;
@@ -483,8 +485,8 @@ int Pars_At_Given_Edge(t_edge *b, t_tree *tree)
 
 int Update_Pars_At_Given_Edge(t_edge *b_fcus, t_tree *tree)
 {
-  Update_P_Pars(tree,b_fcus,b_fcus->left);
-  Update_P_Pars(tree,b_fcus,b_fcus->rght);
+  Update_Partial_Pars(tree,b_fcus,b_fcus->left);
+  Update_Partial_Pars(tree,b_fcus,b_fcus->rght);
   tree->c_pars = Pars(b_fcus,tree);
   return tree->c_pars;
 }
@@ -898,7 +900,7 @@ void Get_Step_Mat(t_tree *tree)
       tree->step_mat[19*tree->mod->ns+18] =    3 ;
       tree->step_mat[19*tree->mod->ns+19] =    0 ;
     }
-  else
+  else if(tree->io->datatype == NT)
     {
       tree->step_mat[0*tree->mod->ns+0] = 0;
       tree->step_mat[0*tree->mod->ns+1] = 2;
@@ -919,32 +921,155 @@ void Get_Step_Mat(t_tree *tree)
       tree->step_mat[3*tree->mod->ns+1] = 1;
       tree->step_mat[3*tree->mod->ns+2] = 2;
       tree->step_mat[3*tree->mod->ns+3] = 0;
-
-/*       tree->step_mat[0*tree->mod->ns+0] = 0; */
-/*       tree->step_mat[0*tree->mod->ns+1] = 1; */
-/*       tree->step_mat[0*tree->mod->ns+2] = 1; */
-/*       tree->step_mat[0*tree->mod->ns+3] = 1; */
-
-/*       tree->step_mat[1*tree->mod->ns+0] = 1; */
-/*       tree->step_mat[1*tree->mod->ns+1] = 0; */
-/*       tree->step_mat[1*tree->mod->ns+2] = 1; */
-/*       tree->step_mat[1*tree->mod->ns+3] = 1; */
-
-/*       tree->step_mat[2*tree->mod->ns+0] = 1; */
-/*       tree->step_mat[2*tree->mod->ns+1] = 1; */
-/*       tree->step_mat[2*tree->mod->ns+2] = 0; */
-/*       tree->step_mat[2*tree->mod->ns+3] = 1; */
-
-/*       tree->step_mat[3*tree->mod->ns+0] = 1; */
-/*       tree->step_mat[3*tree->mod->ns+1] = 1; */
-/*       tree->step_mat[3*tree->mod->ns+2] = 1; */
-/*       tree->step_mat[3*tree->mod->ns+3] = 0; */
-
+    }
+  else if(tree->io->datatype == GENERIC)
+    {
+      int j;
+      for(i=0;i<tree->mod->ns;i++)
+        for(j=0;j<tree->mod->ns;j++)
+          if(j!=i) tree->step_mat[i*tree->mod->ns+j] = 1;
     }
 
-  For(i,tree->mod->ns) tree->step_mat[i*tree->mod->ns+i] = 0;
+  for(i=0;i<tree->mod->ns;i++) tree->step_mat[i*tree->mod->ns+i] = 0;
 }
 
 /*********************************************************/
-
 /*********************************************************/
+/*********************************************************/
+/*********************************************************/
+/*********************************************************/
+/*********************************************************/
+/*********************************************************/
+
+// Tree should be ready for likelihood analysis when calling
+// this function.
+void Stepwise_Add_Pars(t_tree *tree)
+{
+  t_edge **residuals,**targets,*best_target;
+  int *nd_idx,i,j,n_targets,*tg_idx;
+
+  residuals   = (t_edge **)mCalloc(tree->n_otu-3,sizeof(t_edge *));
+  targets     = (t_edge **)mCalloc(2*tree->n_otu-3,sizeof(t_edge *));
+  best_target = NULL;
+  nd_idx      = Permutate(tree->n_otu-3);
+
+  // Remove all tips except that corresponding to a_nodes[0], 
+  // a_nodes[1] and a_nodes[2].  
+  for(i=0;i<tree->n_otu-3;++i)
+    {
+      Prune_Subtree(tree->a_nodes[i+3]->v[0],                   
+                    tree->a_nodes[i+3],
+                    NULL,
+                    residuals+i,
+                    tree);
+    }
+
+  // Initial targets
+  n_targets = 3;
+  for(i=0;i<n_targets;i++) targets[i] = tree->a_nodes[i]->b[0];
+
+  // Regraft each tip on the tree at most parsimonious position
+  for(i=0;i<tree->n_otu-3;++i)
+    {
+      Set_Both_Sides(YES,tree);
+      Pars(NULL,tree);
+
+      /* printf("\n. [%d/%d]",i,tree->n_otu-3); */
+
+      tree->best_pars = 1E+8;
+      best_target = NULL;
+      tg_idx      = Permutate(n_targets);
+
+      for(j=0;j<n_targets;j++)
+        {
+          Graft_Subtree(targets[tg_idx[j]],
+                        tree->a_nodes[nd_idx[i]+3]->v[0],
+                        NULL,
+                        residuals[i],
+                        NULL,
+                        tree);
+          
+          Update_Partial_Pars(tree,
+                        tree->a_nodes[nd_idx[i]+3]->b[0],
+                        tree->a_nodes[nd_idx[i]+3]->v[0]);
+          Pars(tree->a_nodes[nd_idx[i]+3]->b[0],tree);
+          
+
+          if(tree->c_pars < tree->best_pars)
+            {
+              tree->best_pars = tree->c_pars;
+              best_target = targets[tg_idx[j]];
+            }
+          
+          Prune_Subtree(tree->a_nodes[nd_idx[i]+3]->v[0],                        
+                        tree->a_nodes[nd_idx[i]+3],
+                        NULL,
+                        residuals+i,
+                        tree);
+        }
+
+      assert(best_target);
+            
+      Graft_Subtree(best_target,
+                    tree->a_nodes[nd_idx[i]+3]->v[0],
+                    NULL,
+                    residuals[i],
+                    NULL,
+                    tree);
+      
+      targets[n_targets]   = residuals[i]; 
+      targets[n_targets+1] = tree->a_nodes[nd_idx[i]+3]->b[0];
+      
+      Free(tg_idx);
+      n_targets+=2;
+    }
+
+  /* printf("\n. pars: %d",tree->c_pars); */
+  /* Exit("\n"); */
+
+  Free(nd_idx);
+  Free(residuals);
+  Free(targets);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
